/*-----------------------------------------------------------------------

File Name: ArrayFactories.c

<lalVerbatim file="ArrayFactoriesCV">
Revision: $Id$
</lalVerbatim>

-------------------------------------------------------------------------*/

/* <lalLaTeX>

\subsection{Module \texttt{ArrayFactories.c}}
\label{ss:ArrayFactories.c}

Create/destroy $\langle\mbox{datatype}\rangle$Array objects.

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{ArrayFactoriesD}
\idx{LALZCreateArray()}
\idx{LALCCreateArray()}
\idx{LALDCreateArray()}
\idx{LALSCreateArray()}
\idx{LALI2CreateArray()}
\idx{LALI4CreateArray()}
\idx{LALI8CreateArray()}
\idx{LALU2CreateArray()}
\idx{LALU4CreateArray()}
\idx{LALU8CreateArray()}
\idx{LALCreateArray()}
\idx{LALZDestroyArray()}
\idx{LALCDestroyArray()}
\idx{LALDDestroyArray()}
\idx{LALSDestroyArray()}
\idx{LALI2DestroyArray()}
\idx{LALI4DestroyArray()}
\idx{LALI8DestroyArray()}
\idx{LALU2DestroyArray()}
\idx{LALU4DestroyArray()}
\idx{LALU8DestroyArray()}
\idx{LALDestroyArray()}

\subsubsection*{Description}

The \texttt{CreateArray} family of functions create a
$\langle\mbox{datatype}\rangle$\texttt{Array} of the appropriate dimensions.

The \texttt{DestroyArray} family of functions return the storage allocated by
the \texttt{CreateArray} functions to the system.

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
LALMalloc()
LALFree()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{ArrayFactoriesCV}}

</lalLaTeX> */

#include <string.h>
#include "LALStdlib.h"
#include "AVFactories.h"

/* <lalVerbatim file="ArrayFactoriesNRCSID"> */
NRCSID( ARRAYFACTORIESC, "$Id$" );
/* </lalVerbatim> */















 
 
 

COMPLEX16Array * XLALCreateCOMPLEX16ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  COMPLEX16Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateCOMPLEX16ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16ArrayL", XLAL_EFUNC );
  return arr;
}

COMPLEX16Array * XLALCreateCOMPLEX16ArrayV ( UINT4 ndim, UINT4 *dims )
{
  COMPLEX16Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateCOMPLEX16Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16ArrayV", XLAL_EFUNC );
  return arr;
}


COMPLEX16Array * XLALCreateCOMPLEX16Array ( UINT4Vector *dimLength )
{
  COMPLEX16Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX16Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALZCreateArray ( LALStatus *status, COMPLEX16Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALZCreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateCOMPLEX16Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


COMPLEX16Array * XLALResizeCOMPLEX16ArrayL ( COMPLEX16Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyCOMPLEX16Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX16ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeCOMPLEX16ArrayV ( array, ndim, dims );
}

COMPLEX16Array * XLALResizeCOMPLEX16ArrayV ( COMPLEX16Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyCOMPLEX16Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX16ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeCOMPLEX16Array ( array, &dimLength );
}

COMPLEX16Array * XLALResizeCOMPLEX16Array ( COMPLEX16Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateCOMPLEX16Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyCOMPLEX16Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX16Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX16Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX16Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeCOMPLEX16Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX16Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALZResizeArray ( LALStatus *status, COMPLEX16Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  COMPLEX16Array *tmparr = NULL;

  INITSTATUS (status, "LALZResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateCOMPLEX16Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyCOMPLEX16Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeCOMPLEX16Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyCOMPLEX16Array ( COMPLEX16Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyCOMPLEX16Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyCOMPLEX16Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALZDestroyArray ( LALStatus *status, COMPLEX16Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALZDestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyCOMPLEX16Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}
















 
 
 

COMPLEX8Array * XLALCreateCOMPLEX8ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  COMPLEX8Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateCOMPLEX8ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8ArrayL", XLAL_EFUNC );
  return arr;
}

COMPLEX8Array * XLALCreateCOMPLEX8ArrayV ( UINT4 ndim, UINT4 *dims )
{
  COMPLEX8Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateCOMPLEX8Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8ArrayV", XLAL_EFUNC );
  return arr;
}


COMPLEX8Array * XLALCreateCOMPLEX8Array ( UINT4Vector *dimLength )
{
  COMPLEX8Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateCOMPLEX8Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALCCreateArray ( LALStatus *status, COMPLEX8Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALCCreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateCOMPLEX8Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


COMPLEX8Array * XLALResizeCOMPLEX8ArrayL ( COMPLEX8Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyCOMPLEX8Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX8ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeCOMPLEX8ArrayV ( array, ndim, dims );
}

COMPLEX8Array * XLALResizeCOMPLEX8ArrayV ( COMPLEX8Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyCOMPLEX8Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX8ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeCOMPLEX8Array ( array, &dimLength );
}

COMPLEX8Array * XLALResizeCOMPLEX8Array ( COMPLEX8Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateCOMPLEX8Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyCOMPLEX8Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX8Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX8Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX8Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeCOMPLEX8Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeCOMPLEX8Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALCResizeArray ( LALStatus *status, COMPLEX8Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  COMPLEX8Array *tmparr = NULL;

  INITSTATUS (status, "LALCResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateCOMPLEX8Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyCOMPLEX8Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeCOMPLEX8Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyCOMPLEX8Array ( COMPLEX8Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyCOMPLEX8Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyCOMPLEX8Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALCDestroyArray ( LALStatus *status, COMPLEX8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALCDestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyCOMPLEX8Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}
















 
 
 

REAL8Array * XLALCreateREAL8ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  REAL8Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateREAL8ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateREAL8ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateREAL8ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateREAL8ArrayL", XLAL_EFUNC );
  return arr;
}

REAL8Array * XLALCreateREAL8ArrayV ( UINT4 ndim, UINT4 *dims )
{
  REAL8Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateREAL8ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateREAL8ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateREAL8Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateREAL8ArrayV", XLAL_EFUNC );
  return arr;
}


REAL8Array * XLALCreateREAL8Array ( UINT4Vector *dimLength )
{
  REAL8Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateREAL8Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateREAL8Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateREAL8Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateREAL8Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateREAL8Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateREAL8Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateREAL8Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALDCreateArray ( LALStatus *status, REAL8Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALDCreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateREAL8Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


REAL8Array * XLALResizeREAL8ArrayL ( REAL8Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyREAL8Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeREAL8ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeREAL8ArrayV ( array, ndim, dims );
}

REAL8Array * XLALResizeREAL8ArrayV ( REAL8Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyREAL8Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeREAL8ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeREAL8Array ( array, &dimLength );
}

REAL8Array * XLALResizeREAL8Array ( REAL8Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateREAL8Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyREAL8Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeREAL8Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeREAL8Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeREAL8Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeREAL8Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeREAL8Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALDResizeArray ( LALStatus *status, REAL8Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  REAL8Array *tmparr = NULL;

  INITSTATUS (status, "LALDResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateREAL8Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyREAL8Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeREAL8Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyREAL8Array ( REAL8Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyREAL8Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyREAL8Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALDDestroyArray ( LALStatus *status, REAL8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALDDestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyREAL8Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}
















 
 
 

REAL4Array * XLALCreateREAL4ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  REAL4Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateREAL4ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateREAL4ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateREAL4ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateREAL4ArrayL", XLAL_EFUNC );
  return arr;
}

REAL4Array * XLALCreateREAL4ArrayV ( UINT4 ndim, UINT4 *dims )
{
  REAL4Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateREAL4ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateREAL4ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateREAL4Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateREAL4ArrayV", XLAL_EFUNC );
  return arr;
}


REAL4Array * XLALCreateREAL4Array ( UINT4Vector *dimLength )
{
  REAL4Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateREAL4Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateREAL4Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateREAL4Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateREAL4Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateREAL4Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateREAL4Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateREAL4Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALSCreateArray ( LALStatus *status, REAL4Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALSCreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateREAL4Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


REAL4Array * XLALResizeREAL4ArrayL ( REAL4Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyREAL4Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeREAL4ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeREAL4ArrayV ( array, ndim, dims );
}

REAL4Array * XLALResizeREAL4ArrayV ( REAL4Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyREAL4Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeREAL4ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeREAL4Array ( array, &dimLength );
}

REAL4Array * XLALResizeREAL4Array ( REAL4Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateREAL4Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyREAL4Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeREAL4Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeREAL4Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeREAL4Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeREAL4Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeREAL4Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALSResizeArray ( LALStatus *status, REAL4Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  REAL4Array *tmparr = NULL;

  INITSTATUS (status, "LALSResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateREAL4Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyREAL4Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeREAL4Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyREAL4Array ( REAL4Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyREAL4Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyREAL4Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALSDestroyArray ( LALStatus *status, REAL4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALSDestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyREAL4Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}
















 
 
 

INT2Array * XLALCreateINT2ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  INT2Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateINT2ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateINT2ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateINT2ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateINT2ArrayL", XLAL_EFUNC );
  return arr;
}

INT2Array * XLALCreateINT2ArrayV ( UINT4 ndim, UINT4 *dims )
{
  INT2Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateINT2ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateINT2ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateINT2Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateINT2ArrayV", XLAL_EFUNC );
  return arr;
}


INT2Array * XLALCreateINT2Array ( UINT4Vector *dimLength )
{
  INT2Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateINT2Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateINT2Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateINT2Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateINT2Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateINT2Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateINT2Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateINT2Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI2CreateArray ( LALStatus *status, INT2Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALI2CreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateINT2Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


INT2Array * XLALResizeINT2ArrayL ( INT2Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyINT2Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeINT2ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeINT2ArrayV ( array, ndim, dims );
}

INT2Array * XLALResizeINT2ArrayV ( INT2Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyINT2Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeINT2ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeINT2Array ( array, &dimLength );
}

INT2Array * XLALResizeINT2Array ( INT2Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateINT2Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyINT2Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeINT2Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeINT2Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeINT2Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeINT2Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeINT2Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI2ResizeArray ( LALStatus *status, INT2Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  INT2Array *tmparr = NULL;

  INITSTATUS (status, "LALI2ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateINT2Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyINT2Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeINT2Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyINT2Array ( INT2Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyINT2Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyINT2Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI2DestroyArray ( LALStatus *status, INT2Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALI2DestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyINT2Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}
















 
 
 

INT4Array * XLALCreateINT4ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  INT4Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateINT4ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateINT4ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateINT4ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateINT4ArrayL", XLAL_EFUNC );
  return arr;
}

INT4Array * XLALCreateINT4ArrayV ( UINT4 ndim, UINT4 *dims )
{
  INT4Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateINT4ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateINT4ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateINT4Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateINT4ArrayV", XLAL_EFUNC );
  return arr;
}


INT4Array * XLALCreateINT4Array ( UINT4Vector *dimLength )
{
  INT4Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateINT4Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateINT4Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateINT4Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateINT4Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateINT4Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateINT4Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateINT4Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI4CreateArray ( LALStatus *status, INT4Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALI4CreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateINT4Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


INT4Array * XLALResizeINT4ArrayL ( INT4Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyINT4Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeINT4ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeINT4ArrayV ( array, ndim, dims );
}

INT4Array * XLALResizeINT4ArrayV ( INT4Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyINT4Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeINT4ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeINT4Array ( array, &dimLength );
}

INT4Array * XLALResizeINT4Array ( INT4Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateINT4Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyINT4Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeINT4Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeINT4Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeINT4Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeINT4Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeINT4Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI4ResizeArray ( LALStatus *status, INT4Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  INT4Array *tmparr = NULL;

  INITSTATUS (status, "LALI4ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateINT4Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyINT4Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeINT4Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyINT4Array ( INT4Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyINT4Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyINT4Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI4DestroyArray ( LALStatus *status, INT4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALI4DestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyINT4Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}
















 
 
 

INT8Array * XLALCreateINT8ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  INT8Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateINT8ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateINT8ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateINT8ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateINT8ArrayL", XLAL_EFUNC );
  return arr;
}

INT8Array * XLALCreateINT8ArrayV ( UINT4 ndim, UINT4 *dims )
{
  INT8Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateINT8ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateINT8ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateINT8Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateINT8ArrayV", XLAL_EFUNC );
  return arr;
}


INT8Array * XLALCreateINT8Array ( UINT4Vector *dimLength )
{
  INT8Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateINT8Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateINT8Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateINT8Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateINT8Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateINT8Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateINT8Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateINT8Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI8CreateArray ( LALStatus *status, INT8Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALI8CreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateINT8Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


INT8Array * XLALResizeINT8ArrayL ( INT8Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyINT8Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeINT8ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeINT8ArrayV ( array, ndim, dims );
}

INT8Array * XLALResizeINT8ArrayV ( INT8Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyINT8Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeINT8ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeINT8Array ( array, &dimLength );
}

INT8Array * XLALResizeINT8Array ( INT8Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateINT8Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyINT8Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeINT8Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeINT8Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeINT8Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeINT8Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeINT8Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI8ResizeArray ( LALStatus *status, INT8Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  INT8Array *tmparr = NULL;

  INITSTATUS (status, "LALI8ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateINT8Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyINT8Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeINT8Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyINT8Array ( INT8Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyINT8Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyINT8Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI8DestroyArray ( LALStatus *status, INT8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALI8DestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyINT8Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}
















 
 
 

UINT2Array * XLALCreateUINT2ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT2Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateUINT2ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateUINT2ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateUINT2ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateUINT2ArrayL", XLAL_EFUNC );
  return arr;
}

UINT2Array * XLALCreateUINT2ArrayV ( UINT4 ndim, UINT4 *dims )
{
  UINT2Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateUINT2ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateUINT2ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateUINT2Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateUINT2ArrayV", XLAL_EFUNC );
  return arr;
}


UINT2Array * XLALCreateUINT2Array ( UINT4Vector *dimLength )
{
  UINT2Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateUINT2Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateUINT2Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateUINT2Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateUINT2Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateUINT2Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateUINT2Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateUINT2Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU2CreateArray ( LALStatus *status, UINT2Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALU2CreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateUINT2Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


UINT2Array * XLALResizeUINT2ArrayL ( UINT2Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyUINT2Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeUINT2ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeUINT2ArrayV ( array, ndim, dims );
}

UINT2Array * XLALResizeUINT2ArrayV ( UINT2Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyUINT2Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeUINT2ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeUINT2Array ( array, &dimLength );
}

UINT2Array * XLALResizeUINT2Array ( UINT2Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateUINT2Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyUINT2Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeUINT2Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeUINT2Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeUINT2Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeUINT2Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeUINT2Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU2ResizeArray ( LALStatus *status, UINT2Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT2Array *tmparr = NULL;

  INITSTATUS (status, "LALU2ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateUINT2Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyUINT2Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeUINT2Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyUINT2Array ( UINT2Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyUINT2Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyUINT2Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU2DestroyArray ( LALStatus *status, UINT2Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALU2DestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyUINT2Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}
















 
 
 

UINT4Array * XLALCreateUINT4ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateUINT4ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateUINT4ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateUINT4ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateUINT4ArrayL", XLAL_EFUNC );
  return arr;
}

UINT4Array * XLALCreateUINT4ArrayV ( UINT4 ndim, UINT4 *dims )
{
  UINT4Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateUINT4ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateUINT4ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateUINT4Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateUINT4ArrayV", XLAL_EFUNC );
  return arr;
}


UINT4Array * XLALCreateUINT4Array ( UINT4Vector *dimLength )
{
  UINT4Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateUINT4Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateUINT4Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateUINT4Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateUINT4Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateUINT4Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateUINT4Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateUINT4Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU4CreateArray ( LALStatus *status, UINT4Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALU4CreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateUINT4Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


UINT4Array * XLALResizeUINT4ArrayL ( UINT4Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyUINT4Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeUINT4ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeUINT4ArrayV ( array, ndim, dims );
}

UINT4Array * XLALResizeUINT4ArrayV ( UINT4Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyUINT4Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeUINT4ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeUINT4Array ( array, &dimLength );
}

UINT4Array * XLALResizeUINT4Array ( UINT4Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateUINT4Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyUINT4Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeUINT4Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeUINT4Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeUINT4Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeUINT4Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeUINT4Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU4ResizeArray ( LALStatus *status, UINT4Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4Array *tmparr = NULL;

  INITSTATUS (status, "LALU4ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateUINT4Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyUINT4Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeUINT4Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyUINT4Array ( UINT4Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyUINT4Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyUINT4Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU4DestroyArray ( LALStatus *status, UINT4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALU4DestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyUINT4Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}
















 
 
 

UINT8Array * XLALCreateUINT8ArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT8Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateUINT8ArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateUINT8ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateUINT8ArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateUINT8ArrayL", XLAL_EFUNC );
  return arr;
}

UINT8Array * XLALCreateUINT8ArrayV ( UINT4 ndim, UINT4 *dims )
{
  UINT8Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateUINT8ArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateUINT8ArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateUINT8Array ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateUINT8ArrayV", XLAL_EFUNC );
  return arr;
}


UINT8Array * XLALCreateUINT8Array ( UINT4Vector *dimLength )
{
  UINT8Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateUINT8Array", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateUINT8Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateUINT8Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateUINT8Array", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateUINT8Array", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateUINT8Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateUINT8Array", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU8CreateArray ( LALStatus *status, UINT8Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALU8CreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateUINT8Array ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}
















 
 
 
 
 


UINT8Array * XLALResizeUINT8ArrayL ( UINT8Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyUINT8Array ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeUINT8ArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeUINT8ArrayV ( array, ndim, dims );
}

UINT8Array * XLALResizeUINT8ArrayV ( UINT8Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyUINT8Array ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeUINT8ArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeUINT8Array ( array, &dimLength );
}

UINT8Array * XLALResizeUINT8Array ( UINT8Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateUINT8Array ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyUINT8Array ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeUINT8Array", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeUINT8Array", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeUINT8Array", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeUINT8Array", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeUINT8Array", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU8ResizeArray ( LALStatus *status, UINT8Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT8Array *tmparr = NULL;

  INITSTATUS (status, "LALU8ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateUINT8Array ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyUINT8Array ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeUINT8Array ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}














 

void XLALDestroyUINT8Array ( UINT8Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyUINT8Array", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyUINT8Array", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU8DestroyArray ( LALStatus *status, UINT8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALU8DestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyUINT8Array ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}




















REAL4Array * XLALCreateArrayL ( UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  REAL4Array *arr;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateArrayL", XLAL_EBADLEN );
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALCreateArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  arr = XLALCreateArrayV ( ndim, dims );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateArrayL", XLAL_EFUNC );
  return arr;
}

REAL4Array * XLALCreateArrayV ( UINT4 ndim, UINT4 *dims )
{
  REAL4Array *arr;
  UINT4Vector dimLength;

  if ( ! ndim )
    XLAL_ERROR_NULL( "XLALCreateArrayV", XLAL_EBADLEN );
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALCreateArrayV", XLAL_EFAULT );

  dimLength.length = ndim;
  dimLength.data   = dims;

  arr = XLALCreateArray ( &dimLength );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateArrayV", XLAL_EFUNC );
  return arr;
}


REAL4Array * XLALCreateArray ( UINT4Vector *dimLength )
{
  REAL4Array *arr;
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! dimLength )
    XLAL_ERROR_NULL( "XLALCreateArray", XLAL_EFAULT );
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALCreateArray", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALCreateArray", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALCreateArray", XLAL_EBADLEN );

  /* create array */
  arr = LALMalloc( sizeof( *arr ) );
  if ( ! arr )
    XLAL_ERROR_NULL( "XLALCreateArray", XLAL_ENOMEM );

  /* create array dimensions */
  arr->dimLength = XLALCreateUINT4Vector( ndim );
  if ( ! arr->dimLength )
  {
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateArray", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( arr->dimLength->data, dimLength->data,
      ndim * sizeof( *arr->dimLength->data ) );

  /* allocate data storage */
  arr->data = LALMalloc( size * sizeof( *arr->data ) );
  if ( ! arr->data )
  {
    XLALDestroyUINT4Vector( arr->dimLength );
    LALFree( arr );
    XLAL_ERROR_NULL( "XLALCreateArray", XLAL_ENOMEM );
  }

  return arr;
}


/* <lalVerbatim file="ArrayFactoriesD"> */
void LALCreateArray ( LALStatus *status, REAL4Array **array, UINT4Vector *dimLength )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALCreateArray", ARRAYFACTORIESC);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  *array = XLALCreateArray ( dimLength );
  if ( ! *array )
  {
    int code = xlalErrno & ~XLAL_EFUNC; /* turn off subfunction error bit */
    XLALClearErrno();
    if ( code & XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  RETURN (status);
}























REAL4Array * XLALResizeArrayL ( REAL4Array *array, UINT4 ndim, ... )
{
  enum { maxdim = 16 };
  va_list ap;
  UINT4 dims[maxdim];
  UINT4 dim;

  if ( ! ndim )
  {
    XLALDestroyArray ( array );
    return NULL;
  }
  if ( ndim > maxdim )
    XLAL_ERROR_NULL( "XLALResizeArrayL", XLAL_EINVAL );

  va_start( ap, ndim );
  for ( dim = 0; dim < ndim; ++dim )
    dims[dim] = va_arg( ap, UINT4 );
  va_end( ap );

  return XLALResizeArrayV ( array, ndim, dims );
}

REAL4Array * XLALResizeArrayV ( REAL4Array *array, UINT4 ndim, UINT4 *dims )
{
  UINT4Vector dimLength;

  if ( ! ndim )
  {
    XLALDestroyArray ( array );
    return NULL;
  }
  if ( ! dims )
    XLAL_ERROR_NULL( "XLALResizeArrayV", XLAL_EINVAL );

  dimLength.length = ndim;
  dimLength.data   = dims;

  return XLALResizeArray ( array, &dimLength );
}

REAL4Array * XLALResizeArray ( REAL4Array *array, UINT4Vector *dimLength )
{
  UINT4 size = 1;
  UINT4 ndim;
  UINT4 dim;

  if ( ! array )
    return XLALCreateArray ( dimLength );
  if ( ! dimLength )
  {
    XLALDestroyArray ( array );
    return NULL;
  }
  if ( ! dimLength->length )
    XLAL_ERROR_NULL( "XLALResizeArray", XLAL_EBADLEN );
  if ( ! dimLength->data )
    XLAL_ERROR_NULL( "XLALResizeArray", XLAL_EINVAL );

  ndim = dimLength->length;
  for ( dim = 0; dim < ndim; ++dim )
    size *= dimLength->data[dim];

  if ( ! size )
    XLAL_ERROR_NULL( "XLALResizeArray", XLAL_EBADLEN );

  /* resize array->dimLength vector if needed */
  if ( array->dimLength->length != ndim )
  {
    array->dimLength = XLALResizeUINT4Vector( array->dimLength, ndim );
    if ( ! array->dimLength )
      XLAL_ERROR_NULL( "XLALResizeArray", XLAL_EFUNC );
  }

  /* copy dimension lengths */
  memcpy( array->dimLength->data, dimLength->data,
      ndim * sizeof( *array->dimLength->data ) );

  /* reallocate data storage */
  array->data = LALRealloc( array->data, size * sizeof( *array->data ) );
  if ( ! array->data )
    XLAL_ERROR_NULL( "XLALResizeArray", XLAL_ENOMEM );

  return array;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALResizeArray ( LALStatus *status, REAL4Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  REAL4Array *tmparr = NULL;

  INITSTATUS (status, "LALResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( ! array )
  {
    tmparr = XLALCreateArray ( dimLength );
  }
  else if ( ! dimLength )
  {
    XLALDestroyArray ( *array );
    *array = NULL;
  }
  else
  {
    tmparr = XLALResizeArray ( *array, dimLength );
  }

  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
    }
    if ( code == XLAL_EBADLEN )
    {
      ABORT (status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);
    }
    if ( code == XLAL_ENOMEM )
    {
      ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
    }
  }

  *array = tmparr;

  RETURN (status);
}
















void XLALDestroyArray ( REAL4Array *array )
{
  if ( ! array )
    XLAL_ERROR_VOID( "XLALDestroyArray", XLAL_EFAULT );
  if ( ! array->dimLength
      || ! array->dimLength->length
      || ! array->dimLength->data
      || ! array->data )
    XLAL_ERROR_VOID( "XLALDestroyArray", XLAL_EINVAL );
  XLALDestroyUINT4Vector( array->dimLength );
  LALFree( array->data );
  LALFree( array );
  return;
}

/* <lalVerbatim file="ArrayFactoriesD"> */
void LALDestroyArray ( LALStatus *status, REAL4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALDestroyArray", ARRAYFACTORIESC);

  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  XLALDestroyArray ( *array );
  if ( xlalErrno )
  {
    int code = xlalErrno;
    XLALClearErrno();
    if ( code == XLAL_EFAULT )
    {
      ABORT (status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
    }
    if ( code == XLAL_EINVAL )
    {
      ABORT (status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);
    }
  }
  *array = NULL;	    /* make sure we don't point to freed struct */

  RETURN (status);
}

