/************************************ <lalVerbatim file="LALMomentCV">
Author: Tibbits, M M
$Id$
************************************* </lalVerbatim> */

/********************************************************** <lalLaTeX>
\subsection{Module \texttt{LALMoment.c}}
\label{s:LALMoment.c}

Routine to compute various moments of data.

\subsubsection*{Prototypes}
\input{LALMomentCP}

\subsubsection*{Description}
The data structure passed in is either a REAL8 or a REAL4 Sequence.  The only parameter is which moment to calculate.
The function the sums the data, calculates the average and then it returns the average for the first moment, it returns the variance for the second moment, and it returns the n-th moment about the mean for higher order moments.

\subsubsection*{Algorithm}
\begin{itemize}
\item \textit{Find the mean (here referred to as $ \overline{x} $).}
\item \textit{Sum, over all the elements, the quantity: \((x[k] - \overline{x})^{n}. \)}
\item \textit{Divide the sum just made by N-1. Call it moment-n}
\item \textit{If n is greater than 2:}
\begin{itemize}
\item \textit{Sum, over all the elements, the quantity: \((x[k] - \overline{x})^{n}. \)}
\item \textit{Divide the sum just made by N. Call it moment-n}
\end{itemize}
\item \textit{Return moment-n}
\end{itemize}

\subsubsection*{Uses}

Determination of a specific moment of a set of data.

\subsubsection*{Notes}

\begin{itemize}
\item \textit{Moments less than two are not allowed.}
\item \textit{The result structure must be Non-NULL when passed in.}
\item \textit{The function assumes that the length member of the data passed in is correct.}
\end{itemize}

\vfill{\footnotesize\input{LALMomentCV}}

******************************************************* </lalLaTeX> */

#include <math.h>
#include <lal/LALMoment.h>

NRCSID( LALMOMENTC, "$Id$");









/* <lalVerbatim file="LALMomentCP"> */
void LALDMoment (
	LALStatus		*status,
	REAL8			*result,
	REAL8Sequence		*data,
	INT4			whichMoment
)
{	/* </lalVerbatim> */

	/*  Variable Declarations  */
	INT4	iterator;
	INT4	length;
	REAL8	ave 	= 0.0;
	REAL8	momentn	= 0.0;
	REAL8	sum	= 0.0;
	REAL8	base	= 0.0;

	INITSTATUS( status, "LALDMoment" , LALMOMENTC);

	/*  Check input for existence.  */
	/*  Result should come in Allocated  */
	ASSERT ( result, status, LALMOMENTH_ENULL, LALMOMENTH_MSGENULL);

	/*  whichMoment must be greater than 1  */
	ASSERT ( whichMoment >= 1, status, LALMOMENTH_ENULL, LALMOMENTH_MSGENULL);

	/*  data must be defined  */
	ASSERT ( data, status, LALMOMENTH_ENULL, LALMOMENTH_MSGENULL);

	length = ((REAL8Sequence*)(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length > 1, status, LALMOMENTH_ELNTH, LALMOMENTH_MSGELNTH);

	for (iterator = 0; iterator < length; iterator++)
	{
		sum += ((REAL8Sequence*)(data))->data[iterator];
	}

	ave = ( sum / length );

	/*  Return the Mean if whichMoment = 1  */
	if ( whichMoment == 1 )
	{
		*result = ave;
		RETURN(status);
	}

	for (iterator = 0; iterator < length; iterator++)
	{
		base = (data->data[iterator] - ave);
		sum = pow( base, whichMoment );
		momentn += sum;
	}

	momentn /= ((REAL8)(length - 1));


	if ( whichMoment > 2 )
	{
		for (iterator = 0; iterator < length; iterator++)
		{
			base = (data->data[iterator] - ave);
			sum = pow( base, whichMoment );
			momentn += sum;
		}

		momentn /= ((REAL8)(length));
	}


	*result = momentn;

	RETURN (status);
}










/* <lalVerbatim file="LALMomentCP"> */
void LALSMoment (
	LALStatus		*status,
	REAL4			*result,
	REAL4Sequence		*data,
	INT4			whichMoment
)
{	/* </lalVerbatim> */

	/*  Variable Declarations  */
	INT4	iterator;
	INT4	length;
	REAL4	ave 	= 0.0;
	REAL4	momentn	= 0.0;
	REAL4	sum	= 0.0;
	REAL4	base	= 0.0;

	INITSTATUS( status, "LALSMoment" , LALMOMENTC);

	/*  Check input for existence.  */
	/*  Result should come in Allocated  */
	ASSERT ( result, status, LALMOMENTH_ENULL, LALMOMENTH_MSGENULL);

	/*  whichMoment must be greater than 1  */
	ASSERT ( whichMoment >= 1, status, LALMOMENTH_ENULL, LALMOMENTH_MSGENULL);

	/*  data must be defined  */
	ASSERT ( data, status, LALMOMENTH_ENULL, LALMOMENTH_MSGENULL);

	length = ((REAL8Sequence*)(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length > 1, status, LALMOMENTH_ELNTH, LALMOMENTH_MSGELNTH);

	for (iterator = 0; iterator < length; iterator++)
	{
		sum += ((REAL4Sequence*)(data))->data[iterator];
	}

	ave = ( sum / length );

	/*  Return the Mean if whichMoment = 1  */
	if ( whichMoment == 1 )
	{
		*result = ave;
		RETURN(status);
	}

	for (iterator = 0; iterator < length; iterator++)
	{
		base = (data->data[iterator] - ave);
		sum = pow( base, whichMoment );
		momentn += sum;
	}

	momentn /= ((REAL4)(length - 1));


	if ( whichMoment > 2 )
	{
		for (iterator = 0; iterator < length; iterator++)
		{
			base = (data->data[iterator] - ave);
			sum = pow( base, whichMoment );
			momentn += sum;
		}

		momentn /= ((REAL4)(length));
	}


	*result = momentn;

	RETURN (status);
}

