/************************************ <lalVerbatim file="MatrixOpsCV">
Author: Creighton, T. D.
$Id$
**************************************************** </lalVerbatim> */

/********************************************************** <lalLaTeX>

\subsection{Module \texttt{MatrixOps.c}}
\label{ss:MatrixOps.c}

Routines to perform basic matrix operations.

\subsubsection*{Prototypes}
\vspace{0.1in}
\begin{verbatim}
void
LAL<typecode>MatrixAdd( LALStatus *stat,
                        <datatype>Array *out,
                        <datatype>Array *in1,
                        <datatype>Array *in2 )

void
LAL<typecode>MatrixMultiply( LALStatus *stat,
                             <datatype>Array *out,
                             <datatype>Array *in1,
                             <datatype>Array *in2 )

void
LAL<typecode>MatrixTranspose( LALStatus *stat,
                              <datatype>Array *out,
                              <datatype>Array *in )
\end{verbatim}
\input{MatrixOpsCP}

\idx{LALI2MatrixAdd()}
\idx{LALI4MatrixAdd()}
\idx{LALI8MatrixAdd()}
\idx{LALU2MatrixAdd()}
\idx{LALU4MatrixAdd()}
\idx{LALU8MatrixAdd()}
\idx{LALSMatrixAdd()}
\idx{LALDMatrixAdd()}
\idx{LALCMatrixAdd()}
\idx{LALZMatrixAdd()}

\idx{LALI2MatrixMultiply()}
\idx{LALI4MatrixMultiply()}
\idx{LALI8MatrixMultiply()}
\idx{LALU2MatrixMultiply()}
\idx{LALU4MatrixMultiply()}
\idx{LALU8MatrixMultiply()}
\idx{LALSMatrixMultiply()}
\idx{LALDMatrixMultiply()}
\idx{LALCMatrixMultiply()}
\idx{LALZMatrixMultiply()}

\idx{LALI2MatrixTranspose()}
\idx{LALI4MatrixTranspose()}
\idx{LALI8MatrixTranspose()}
\idx{LALU2MatrixTranspose()}
\idx{LALU4MatrixTranspose()}
\idx{LALU8MatrixTranspose()}
\idx{LALSMatrixTranspose()}
\idx{LALDMatrixTranspose()}
\idx{LALCMatrixTranspose()}
\idx{LALZMatrixTranspose()}

\idx{LALCMatrixAdjoint()}
\idx{LALZMatrixAdjoint()}

\subsubsection*{Description}

The routines \verb@LAL<datatype>MatrixAdd()@ add the matrices
\verb@*in1@ and \verb@*in2@ element-by-element, storing the result in
\verb@*out@.  All of these matrices must have the same dimensionality.
The addition may be performed in-place by pointing \verb@out@ to the
same structure as either \verb@in1@ or \verb@in2@.

The routines \verb@LAL<datatype>MatrixMultiply()@ perform matrix
multiplication, contracting the columns of \verb@*in1@ against the
rows of \verb@*in2@, and storing the result in \verb@*out@.  The
number of columns of \verb@*in1@ must equal the number of rows of
\verb@*in2@, and \verb@*out@ must have the same number of columns as
\verb@*in1@ and the same number of rows as \verb@*in2@.

The routines \verb@LAL<datatype>MatrixTranspose()@ take the transpose
of the matrix \verb@*in@ and store the result in \verb@*out@.  The
number of rows of \verb@*out@ must equal the number of columns of
\verb@*in@, and vice-versa.

The routines \verb@LALCMatrixAdjoint()@ and \verb@LALZMatrixAdjoint()@
take the Hermitian conjugate (adjoint) of the matrix \verb@*in@ and
store the result in \verb@*out@: this involves transposing the matrix
and taking the complex conjugate.  The number of rows of \verb@*out@
must equal the number of columns of \verb@*in@, and vice-versa.

Except for the adjoint routines, the prototype templates above in fact
refer to 10 separate routines each, corresponding to all the numerical
atomic datatypes \verb@<datatype>@ referred to by \verb@<typecode>@:
\begin{center}
\begin{tabular}{|c@{\qquad}c|c@{\qquad}c|}
\hline
\tt <typecode> & \tt <datatype> & \tt <typecode> & \tt <datatype> \\
\hline
\tt I2 & \tt  INT2 & \tt U2 & \tt    UINT2  \\
\tt I4 & \tt  INT4 & \tt U4 & \tt    UINT4  \\
\tt I8 & \tt  INT8 & \tt U8 & \tt    UINT8  \\
\tt  S & \tt REAL4 & \tt  C & \tt COMPLEX8  \\
\tt  D & \tt REAL8 & \tt  Z & \tt COMPLEX16 \\
\hline
\end{tabular}
\end{center}

\subsubsection*{Algorithm}

Matrix addition is simply carried through element-by-element.  It
involves one addition operation per element of the output matrix.

The matrix product $\mathsf{Z}^a{}_b=\mathsf{X}^a{}_c
\mathsf{Y}^c{}_b$ of two matrices $\mathsf{X}^a{}_c$ and
$\mathsf{Y}^c{}_b$ is given by the element formula
$Z^i{}_j=\sum_{k=1}^N X^i{}_k Y^k{}_j$, where $N$ is the number of
columns of $\mathsf{X}^a{}_c$ \emph{and} the number of rows of
$\mathsf{Y}^c{}_b$.  This can also be used to compute the inner
product of two vectors $\mathsf{x}^a$ and $\mathsf{y}^a$: simply store
the transpose $(\mathsf{x}^T)_a$ as a row vector (single-row matrix)
as the first operand, and $\mathsf{y}^a$ as a column vector
(single-column matrix) as the second operand.  To compute the vector
outer product, simply transpose the second argument rather than the
first.  These computations involve $N$ additions and multiplications
per element of the output matrix.

The transpose $(\mathsf{X}^T){}^a{}_b$ of a matrix $\mathsf{X}^a{}_b$
is given by $(X^T){}^i{}_j=X^j{}_i$.  The adjoint
$(\mathsf{X}^\dag){}^a{}_b$ of a complex matrix $\mathsf{X}^a{}_b$ is
given by $(X^\dag){}^i{}_j=X^j{}_i{}^*$, where ${}^*$ denotes complex
conjugation.  Transposition involves no arithmetic operations, just
one assignment per element of the output.  Conjugation involves one
multiplication (negating the sign of the imaginary part) per element.

\subsubsection*{Uses}

\subsubsection*{Notes}

\vfill{\footnotesize\input{MatrixOpsCV}}

******************************************************* </lalLaTeX> */

#include <stdio.h>
#include <lal/LALStdlib.h>
#include <lal/MatrixUtils.h>

NRCSID( MATRIXOPSC, "$Id$" );

void
LALI2MatrixAdd ( LALStatus *stat, INT2Array *out, INT2Array *in1, INT2Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  INT2 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALI2MatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 0
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALI2MatrixMultiply ( LALStatus *stat, INT2Array *out, INT2Array *in1, INT2Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  INT2 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALI2MatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 0
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 0
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALI2MatrixTranspose ( LALStatus *stat, INT2Array *out, INT2Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  INT2 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALI2MatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 0
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

void
LALI4MatrixAdd ( LALStatus *stat, INT4Array *out, INT4Array *in1, INT4Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  INT4 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALI4MatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 0
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALI4MatrixMultiply ( LALStatus *stat, INT4Array *out, INT4Array *in1, INT4Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  INT4 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALI4MatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 0
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 0
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALI4MatrixTranspose ( LALStatus *stat, INT4Array *out, INT4Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  INT4 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALI4MatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 0
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

void
LALI8MatrixAdd ( LALStatus *stat, INT8Array *out, INT8Array *in1, INT8Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  INT8 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALI8MatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 0
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALI8MatrixMultiply ( LALStatus *stat, INT8Array *out, INT8Array *in1, INT8Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  INT8 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALI8MatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 0
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 0
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALI8MatrixTranspose ( LALStatus *stat, INT8Array *out, INT8Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  INT8 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALI8MatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 0
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

void
LALU2MatrixAdd ( LALStatus *stat, UINT2Array *out, UINT2Array *in1, UINT2Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  UINT2 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALU2MatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 0
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALU2MatrixMultiply ( LALStatus *stat, UINT2Array *out, UINT2Array *in1, UINT2Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  UINT2 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALU2MatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 0
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 0
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALU2MatrixTranspose ( LALStatus *stat, UINT2Array *out, UINT2Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  UINT2 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALU2MatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 0
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

void
LALU4MatrixAdd ( LALStatus *stat, UINT4Array *out, UINT4Array *in1, UINT4Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  UINT4 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALU4MatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 0
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALU4MatrixMultiply ( LALStatus *stat, UINT4Array *out, UINT4Array *in1, UINT4Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  UINT4 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALU4MatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 0
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 0
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALU4MatrixTranspose ( LALStatus *stat, UINT4Array *out, UINT4Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  UINT4 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALU4MatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 0
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

void
LALU8MatrixAdd ( LALStatus *stat, UINT8Array *out, UINT8Array *in1, UINT8Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  UINT8 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALU8MatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 0
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALU8MatrixMultiply ( LALStatus *stat, UINT8Array *out, UINT8Array *in1, UINT8Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  UINT8 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALU8MatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 0
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 0
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALU8MatrixTranspose ( LALStatus *stat, UINT8Array *out, UINT8Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  UINT8 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALU8MatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 0
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

void
LALSMatrixAdd ( LALStatus *stat, REAL4Array *out, REAL4Array *in1, REAL4Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  REAL4 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALSMatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 0
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALSMatrixMultiply ( LALStatus *stat, REAL4Array *out, REAL4Array *in1, REAL4Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  REAL4 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALSMatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 0
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 0
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALSMatrixTranspose ( LALStatus *stat, REAL4Array *out, REAL4Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  REAL4 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALSMatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 0
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

void
LALDMatrixAdd ( LALStatus *stat, REAL8Array *out, REAL8Array *in1, REAL8Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  REAL8 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALDMatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 0
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALDMatrixMultiply ( LALStatus *stat, REAL8Array *out, REAL8Array *in1, REAL8Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  REAL8 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALDMatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 0
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 0
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALDMatrixTranspose ( LALStatus *stat, REAL8Array *out, REAL8Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  REAL8 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALDMatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 0
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

void
LALCMatrixAdd ( LALStatus *stat, COMPLEX8Array *out, COMPLEX8Array *in1, COMPLEX8Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  COMPLEX8 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALCMatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 1
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALCMatrixMultiply ( LALStatus *stat, COMPLEX8Array *out, COMPLEX8Array *in1, COMPLEX8Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  COMPLEX8 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALCMatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 1
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 1
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALCMatrixTranspose ( LALStatus *stat, COMPLEX8Array *out, COMPLEX8Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  COMPLEX8 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALCMatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 1
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

void
LALZMatrixAdd ( LALStatus *stat, COMPLEX16Array *out, COMPLEX16Array *in1, COMPLEX16Array *in2 )
{
  UINT4 ni, nj;                      /* index ranges */
  COMPLEX16 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALZMatrixAdd", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do addition. */
  ni *= nj;
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  while ( ni-- ) {
#if 1
    outData->re = in1Data->re + in2Data->re;
    (outData++)->im = (in1Data++)->im + (in2Data++)->im;
#else
    *(outData++) = *(in1Data++) + *(in2Data++);
#endif
  }
  RETURN( stat );
}


void
LALZMatrixMultiply ( LALStatus *stat, COMPLEX16Array *out, COMPLEX16Array *in1, COMPLEX16Array *in2 )
{
  UINT4 i, j, k, ni, nj, nk;         /* dimension indices and ranges */
  UINT4 ij, ik, kj, in, kn;          /* array indices */
  COMPLEX16 *outData, *in1Data, *in2Data; /* data pointers */

  INITSTATUS( stat, "LALZMatrixMultiply", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in2, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in2->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  nk = in1->dimLength->data[1];
  ASSERT( in1->dimLength->data[0] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[0] == nk, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in2->dimLength->data[1] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do multiplication. */
  outData = out->data;
  in1Data = in1->data;
  in2Data = in2->data;
  for ( i = 0, in = 0, kn = 0; i < ni; i++, in += nj, kn += nk ) {
    for ( j = 0, ij = in; j < nj; j++, ij++ ) {
#if 1
      outData[ij].re = outData[ij].im = 0.0;
#else
      outData[ij] = 0.0;
#endif
      for ( k = 0, ik = kn, kj = j; k < nk; k++, ik++, kj += nj ) {
#if 1
	outData[ij].re += in1Data[ik].re*in2Data[kj].re
	  - in1Data[ik].im*in2Data[kj].im;
	outData[ij].im += in1Data[ik].im*in2Data[kj].re
	  + in1Data[ik].re*in2Data[kj].im;
#else
	outData[ij] += in1Data[ik]*in2Data[kj];
#endif
      }
    }
  }
  RETURN( stat );
}


void
LALZMatrixTranspose ( LALStatus *stat, COMPLEX16Array *out, COMPLEX16Array *in1 )
{
  UINT4 i, j, ni, nj;      /* dimension indices and ranges */
  UINT4 ij, ji, in;        /* array indices */
  COMPLEX16 *outData, *in1Data; /* data pointers */

  INITSTATUS( stat, "LALZMatrixTranspose", MATRIXOPSC );

  /* Check for valid input arguments. */
  ASSERT( out, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( in1, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->data, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength, stat, MATRIXUTILSH_ENUL, MATRIXUTILSH_MSGENUL );
  ASSERT( in1->dimLength->data, stat, MATRIXUTILSH_ENUL,
	  MATRIXUTILSH_MSGENUL );
  ASSERT( out->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->length == 2, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ni = out->dimLength->data[0];
  nj = out->dimLength->data[1];
  ASSERT( in1->dimLength->data[1] == ni, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );
  ASSERT( in1->dimLength->data[0] == nj, stat, MATRIXUTILSH_EDIM,
	  MATRIXUTILSH_MSGEDIM );

  /* Do transposition. */
  outData = out->data;
  in1Data = in1->data;
  for ( i = 0, in = 0; i < ni; i++, in += nj ) {
    for ( j = 0, ij = in, ji = i; j < nj; j++, ij++, ji += ni ) {
#if 1
      outData[ij].re = in1Data[ji].re;
      outData[ij].im = in1Data[ji].im;
#else
      outData[ij] = in1Data[ji];
#endif
    }
  }
  RETURN( stat );
}

/* <lalVerbatim file="MatrixOpsCP"> */
void
LALCMatrixAdjoint( LALStatus *stat, COMPLEX8Array *out, COMPLEX8Array *in1 )
{ /* </lalVerbatim> */
  UINT4 n;        /* number of elements */
  COMPLEX8 *data; /* pointer to elements */

  INITSTATUS( stat, "LALCMatrixAdjoint", MATRIXOPSC );
  ATTATCHSTATUSPTR( stat );

  /* All argument checking is done by the subroutine. */
  TRY( LALCMatrixTranspose( stat->statusPtr, out, in1 ), stat );

  /* We just need to conjugate the result. */
  n = out->dimLength->data[0]*out->dimLength->data[1];
  data = out->data;
  while ( n-- )
    (data++)->im *= -1.0;

  /* That's all. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


/* <lalVerbatim file="MatrixOpsCP"> */
void
LALZMatrixAdjoint( LALStatus *stat, COMPLEX16Array *out, COMPLEX16Array *in1 )
{ /* </lalVerbatim> */
  UINT4 n;         /* number of elements */
  COMPLEX16 *data; /* pointer to elements */

  INITSTATUS( stat, "LALZMatrixAdjoint", MATRIXOPSC );
  ATTATCHSTATUSPTR( stat );

  /* All argument checking is done by the subroutine. */
  TRY( LALZMatrixTranspose( stat->statusPtr, out, in1 ), stat );

  /* We just need to conjugate the result. */
  n = out->dimLength->data[0]*out->dimLength->data[1];
  data = out->data;
  while ( n-- )
    (data++)->im *= -1.0;

  /* That's all. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}
