/******** <lalVerbatim file="MiscMatlabCV"> ********
Author: Tibbits, M. M.
$Id$
********* </lalVerbatim> ********/

/******************************* <lalLaTeX file="MiscMatlabC">
\subsection{Module \texttt{MiscMatlab.c}}
\label{s:MiscMatlab.c}

This file reproduces the last few matlab functions that we needed for our purposes.
It creates useable forms of cumsum, sum, max, and finally an implemenation of the
array addressing in matlab.  Matlab has an easy of inverting a vector, (end: -1: 1)
and the final function, FlipVector returns a result vector that has been flipped in
that same manner.

\subsection*{Description}

This file reproduces the last few matlab functions that we needed for our purposes.
It creates useable forms of cumsum, sum, max, and finally an implemenation of the
array addressing in matlab.  Matlab has an easy of inverting a vector, (end: -1: 1)
and the final function, FlipVector returns a result vector that has been flipped in
that same manner.

\subsection*{Algorithms}

The algorithms are the same as in matlab.  Flip vector was discussed above.  Sum
takes the sum of all of the elements in a vector.  Cum sum takes an input vector:

vector input[25];
vector output[25];

output[0] = input[0];
output[1] = input[0] + input[1];
output[2] = input[0] + input[1] + input[2];

etc

\subsection*{Uses}

\begin{itemize}
\item \texttt{LALDCreateVector}
\end{itemize}

\subsection*{Notes}

At the current time none of the operations have been specified for neither the
complex datatypes nor the unsigned datatypes.

Also, the prototypes are out of order as I have used m4 to create all of the
functions from one codebase.

\subsubsection*{Prototypes}

 *************************************************** </lalLaTeX> */

#include <math.h>
#include "Matrix.h"

NRCSID( MATLABMATRIXSUMC, "$Id$");













/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALDCumSum ( LALStatus *status, REAL8Vector **result, REAL8Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALDCumSum (
	LALStatus		*status,
	REAL8Vector		**result,
	REAL8Vector		*data
)
{
	/*  Variable Declarations  */
	INT4	iterator;
	INT4	myindex;
	INT4	length;

	INITSTATUS( status, "LALDCumSum" , MATLABMATRIXSUMC);
        ATTATCHSTATUSPTR( status );

	/*  Check input for existence.  */
	/*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

	/*  data must be defined  */
	ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = ((REAL8Vector*)(data))->length;

	/*  length must be greater than one  */
	ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	LALDCreateVector( status->statusPtr, result, length);
	CHECKSTATUSPTR( status );

	myindex = 0;

	while ( myindex < length )
	{
		(*result)->data[myindex] = 0;
		for (iterator = -1; iterator < myindex; iterator++)
		{
			(*result)->data[myindex] += ((REAL8Vector*)(data))->data[iterator+1];
		}

		myindex++;
	}

        DETATCHSTATUSPTR( status );
	RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALDSum ( LALStatus *status, REAL8 *result, REAL8Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALDSum (
        LALStatus		*status,
        REAL8			*result,
        REAL8Vector		*data
)
{
	/*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALDSum" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = (( REAL8Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = 0;

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result) += ((REAL8Vector*)(data))->data[iterator];
	}

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALDMax (LALStatus *status, REAL8 *result, REAL8Vector *data, INT4 *myindex )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALDMax (
        LALStatus		*status,
        REAL8			*result,
        REAL8Vector		*data,
	INT4			*myindex
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALDMax" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = (( REAL8Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = ((REAL8Vector*)(data))->data[length - 1];
	(*myindex) = length - 1;

        for (iterator = 0; iterator < length; iterator++)
        {
                if ( ((REAL8Vector*)(data))->data[iterator] > (*result) )
		{
			(*result) = ((REAL8Vector*)(data))->data[iterator];
			(*myindex) = iterator;
		}
        }

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALDFlipVector (LALStatus *status, REAL8Vector **result, REAL8Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALDFlipVector (
        LALStatus		*status,
        REAL8Vector		**result,
        REAL8Vector           *data
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALDFlipVector" , MATLABMATRIXSUMC);
	ATTATCHSTATUSPTR( status );

        /*  Check input for existence.  */
        /*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = ((REAL8Vector*)(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

        LALDCreateVector( status->statusPtr, result, length);

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result)->data[iterator] = ((REAL8Vector*)(data))->data[length - iterator - 1];
	}

	DETATCHSTATUSPTR( status );
        RETURN( status );
}
















/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALSCumSum ( LALStatus *status, REAL4Vector **result, REAL4Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALSCumSum (
	LALStatus		*status,
	REAL4Vector		**result,
	REAL4Vector		*data
)
{
	/*  Variable Declarations  */
	INT4	iterator;
	INT4	myindex;
	INT4	length;

	INITSTATUS( status, "LALSCumSum" , MATLABMATRIXSUMC);
        ATTATCHSTATUSPTR( status );

	/*  Check input for existence.  */
	/*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

	/*  data must be defined  */
	ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = ((REAL4Vector*)(data))->length;

	/*  length must be greater than one  */
	ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	LALSCreateVector( status->statusPtr, result, length);
	CHECKSTATUSPTR( status );

	myindex = 0;

	while ( myindex < length )
	{
		(*result)->data[myindex] = 0;
		for (iterator = -1; iterator < myindex; iterator++)
		{
			(*result)->data[myindex] += ((REAL4Vector*)(data))->data[iterator+1];
		}

		myindex++;
	}

        DETATCHSTATUSPTR( status );
	RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALSSum ( LALStatus *status, REAL4 *result, REAL4Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALSSum (
        LALStatus		*status,
        REAL4			*result,
        REAL4Vector		*data
)
{
	/*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALSSum" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = (( REAL4Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = 0;

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result) += ((REAL4Vector*)(data))->data[iterator];
	}

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALSMax (LALStatus *status, REAL4 *result, REAL4Vector *data, INT4 *myindex )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALSMax (
        LALStatus		*status,
        REAL4			*result,
        REAL4Vector		*data,
	INT4			*myindex
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALSMax" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = (( REAL4Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = ((REAL4Vector*)(data))->data[length - 1];
	(*myindex) = length - 1;

        for (iterator = 0; iterator < length; iterator++)
        {
                if ( ((REAL4Vector*)(data))->data[iterator] > (*result) )
		{
			(*result) = ((REAL4Vector*)(data))->data[iterator];
			(*myindex) = iterator;
		}
        }

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALSFlipVector (LALStatus *status, REAL4Vector **result, REAL4Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALSFlipVector (
        LALStatus		*status,
        REAL4Vector		**result,
        REAL4Vector           *data
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALSFlipVector" , MATLABMATRIXSUMC);
	ATTATCHSTATUSPTR( status );

        /*  Check input for existence.  */
        /*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = ((REAL4Vector*)(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

        LALSCreateVector( status->statusPtr, result, length);

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result)->data[iterator] = ((REAL4Vector*)(data))->data[length - iterator - 1];
	}

	DETATCHSTATUSPTR( status );
        RETURN( status );
}
















/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI2CumSum ( LALStatus *status, INT2Vector **result, INT2Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI2CumSum (
	LALStatus		*status,
	INT2Vector		**result,
	INT2Vector		*data
)
{
	/*  Variable Declarations  */
	INT4	iterator;
	INT4	myindex;
	INT4	length;

	INITSTATUS( status, "LALI2CumSum" , MATLABMATRIXSUMC);
        ATTATCHSTATUSPTR( status );

	/*  Check input for existence.  */
	/*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

	/*  data must be defined  */
	ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = ((INT2Vector*)(data))->length;

	/*  length must be greater than one  */
	ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	LALI2CreateVector( status->statusPtr, result, length);
	CHECKSTATUSPTR( status );

	myindex = 0;

	while ( myindex < length )
	{
		(*result)->data[myindex] = 0;
		for (iterator = -1; iterator < myindex; iterator++)
		{
			(*result)->data[myindex] += ((INT2Vector*)(data))->data[iterator+1];
		}

		myindex++;
	}

        DETATCHSTATUSPTR( status );
	RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI2Sum ( LALStatus *status, INT2 *result, INT2Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI2Sum (
        LALStatus		*status,
        INT2			*result,
        INT2Vector		*data
)
{
	/*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALI2Sum" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = (( INT2Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = 0;

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result) += ((INT2Vector*)(data))->data[iterator];
	}

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI2Max (LALStatus *status, INT2 *result, INT2Vector *data, INT4 *myindex )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI2Max (
        LALStatus		*status,
        INT2			*result,
        INT2Vector		*data,
	INT4			*myindex
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALI2Max" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = (( INT2Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = ((INT2Vector*)(data))->data[length - 1];
	(*myindex) = length - 1;

        for (iterator = 0; iterator < length; iterator++)
        {
                if ( ((INT2Vector*)(data))->data[iterator] > (*result) )
		{
			(*result) = ((INT2Vector*)(data))->data[iterator];
			(*myindex) = iterator;
		}
        }

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI2FlipVector (LALStatus *status, INT2Vector **result, INT2Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI2FlipVector (
        LALStatus		*status,
        INT2Vector		**result,
        INT2Vector           *data
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALI2FlipVector" , MATLABMATRIXSUMC);
	ATTATCHSTATUSPTR( status );

        /*  Check input for existence.  */
        /*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = ((INT2Vector*)(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

        LALI2CreateVector( status->statusPtr, result, length);

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result)->data[iterator] = ((INT2Vector*)(data))->data[length - iterator - 1];
	}

	DETATCHSTATUSPTR( status );
        RETURN( status );
}
















/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI4CumSum ( LALStatus *status, INT4Vector **result, INT4Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI4CumSum (
	LALStatus		*status,
	INT4Vector		**result,
	INT4Vector		*data
)
{
	/*  Variable Declarations  */
	INT4	iterator;
	INT4	myindex;
	INT4	length;

	INITSTATUS( status, "LALI4CumSum" , MATLABMATRIXSUMC);
        ATTATCHSTATUSPTR( status );

	/*  Check input for existence.  */
	/*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

	/*  data must be defined  */
	ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = ((INT4Vector*)(data))->length;

	/*  length must be greater than one  */
	ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	LALI4CreateVector( status->statusPtr, result, length);
	CHECKSTATUSPTR( status );

	myindex = 0;

	while ( myindex < length )
	{
		(*result)->data[myindex] = 0;
		for (iterator = -1; iterator < myindex; iterator++)
		{
			(*result)->data[myindex] += ((INT4Vector*)(data))->data[iterator+1];
		}

		myindex++;
	}

        DETATCHSTATUSPTR( status );
	RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI4Sum ( LALStatus *status, INT4 *result, INT4Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI4Sum (
        LALStatus		*status,
        INT4			*result,
        INT4Vector		*data
)
{
	/*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALI4Sum" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = (( INT4Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = 0;

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result) += ((INT4Vector*)(data))->data[iterator];
	}

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI4Max (LALStatus *status, INT4 *result, INT4Vector *data, INT4 *myindex )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI4Max (
        LALStatus		*status,
        INT4			*result,
        INT4Vector		*data,
	INT4			*myindex
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALI4Max" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = (( INT4Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = ((INT4Vector*)(data))->data[length - 1];
	(*myindex) = length - 1;

        for (iterator = 0; iterator < length; iterator++)
        {
                if ( ((INT4Vector*)(data))->data[iterator] > (*result) )
		{
			(*result) = ((INT4Vector*)(data))->data[iterator];
			(*myindex) = iterator;
		}
        }

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI4FlipVector (LALStatus *status, INT4Vector **result, INT4Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI4FlipVector (
        LALStatus		*status,
        INT4Vector		**result,
        INT4Vector           *data
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALI4FlipVector" , MATLABMATRIXSUMC);
	ATTATCHSTATUSPTR( status );

        /*  Check input for existence.  */
        /*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = ((INT4Vector*)(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

        LALI4CreateVector( status->statusPtr, result, length);

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result)->data[iterator] = ((INT4Vector*)(data))->data[length - iterator - 1];
	}

	DETATCHSTATUSPTR( status );
        RETURN( status );
}
















/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI8CumSum ( LALStatus *status, INT8Vector **result, INT8Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI8CumSum (
	LALStatus		*status,
	INT8Vector		**result,
	INT8Vector		*data
)
{
	/*  Variable Declarations  */
	INT4	iterator;
	INT4	myindex;
	INT4	length;

	INITSTATUS( status, "LALI8CumSum" , MATLABMATRIXSUMC);
        ATTATCHSTATUSPTR( status );

	/*  Check input for existence.  */
	/*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

	/*  data must be defined  */
	ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = ((INT8Vector*)(data))->length;

	/*  length must be greater than one  */
	ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	LALI8CreateVector( status->statusPtr, result, length);
	CHECKSTATUSPTR( status );

	myindex = 0;

	while ( myindex < length )
	{
		(*result)->data[myindex] = 0;
		for (iterator = -1; iterator < myindex; iterator++)
		{
			(*result)->data[myindex] += ((INT8Vector*)(data))->data[iterator+1];
		}

		myindex++;
	}

        DETATCHSTATUSPTR( status );
	RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI8Sum ( LALStatus *status, INT8 *result, INT8Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI8Sum (
        LALStatus		*status,
        INT8			*result,
        INT8Vector		*data
)
{
	/*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALI8Sum" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = (( INT8Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = 0;

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result) += ((INT8Vector*)(data))->data[iterator];
	}

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI8Max (LALStatus *status, INT8 *result, INT8Vector *data, INT4 *myindex )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI8Max (
        LALStatus		*status,
        INT8			*result,
        INT8Vector		*data,
	INT4			*myindex
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALI8Max" , MATLABMATRIXSUMC);

        /*  Check input for existence.  */
        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

	length = (( INT8Vector* )(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

	(*result) = ((INT8Vector*)(data))->data[length - 1];
	(*myindex) = length - 1;

        for (iterator = 0; iterator < length; iterator++)
        {
                if ( ((INT8Vector*)(data))->data[iterator] > (*result) )
		{
			(*result) = ((INT8Vector*)(data))->data[iterator];
			(*myindex) = iterator;
		}
        }

        RETURN( status );
}

/******************************* <lalLaTeX file="MiscMatlabC">
\begin{verbatim}void LALI8FlipVector (LALStatus *status, INT8Vector **result, INT8Vector *data )\end{verbatim}
*************************************************** </lalLaTeX> */

void LALI8FlipVector (
        LALStatus		*status,
        INT8Vector		**result,
        INT8Vector           *data
)
{
        /*  Variable Declarations  */
        INT4    iterator;
        INT4    length;

        INITSTATUS( status, "LALI8FlipVector" , MATLABMATRIXSUMC);
	ATTATCHSTATUSPTR( status );

        /*  Check input for existence.  */
        /*  Result should come in NULL  */
        ASSERT ( result, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);
        ASSERT ( !(*result), status, MATLABMATRIXH_ENNUL, MATLABMATRIXH_MSGENNUL);

        /*  data must be defined  */
        ASSERT ( data, status, MATLABMATRIXH_ENULL, MATLABMATRIXH_MSGENULL);

        length = ((INT8Vector*)(data))->length;

        /*  length must be greater than one  */
        ASSERT ( length >= 1, status, MATLABMATRIXH_ELNTH, MATLABMATRIXH_MSGELNTH);

        LALI8CreateVector( status->statusPtr, result, length);

	for (iterator = 0; iterator < length; iterator++)
	{
		(*result)->data[iterator] = ((INT8Vector*)(data))->data[length - iterator - 1];
	}

	DETATCHSTATUSPTR( status );
        RETURN( status );
}



