/**************************** <lalVerbatim file="StreamSeriesInputCV">
Author: Creighton, T. D.
$Id$
**************************************************** </lalVerbatim> */

/********************************************************** <lalLaTeX>

\subsection{Module \texttt{StreamSeriesInput.c}}
\label{ss:StreamSeriesInput.c}

Converts an input stream into a time or frequency series.

\subsubsection*{Prototypes}
\vspace{0.1in}
\begin{verbatim}
void
LAL<typecode>ReadTSeries( LALStatus            *stat,
                          <datatype>TimeSeries *series,
                          FILE                 *stream )

void
LAL<typecode>ReadTVectorSeries( LALStatus                  *stat,
                                <datatype>TimeVectorSeries *series,
                                FILE                       *stream )

void
LAL<typecode>ReadTArraySeries( LALStatus                 *stat,
                               <datatype>TimeArraySeries *series,
                               FILE                      *stream )

void
LAL<typecode>ReadFSeries( LALStatus                 *stat,
                          <datatype>FrequencySeries *series,
                          FILE                      *stream )
\end{verbatim}

\idx{LALI2ReadTimeSeries()}
\idx{LALI4ReadTimeSeries()}
\idx{LALI8ReadTimeSeries()}
\idx{LALU2ReadTimeSeries()}
\idx{LALU4ReadTimeSeries()}
\idx{LALU8ReadTimeSeries()}
\idx{LALSReadTimeSeries()}
\idx{LALDReadTimeSeries()}
\idx{LALCReadTimeSeries()}
\idx{LALZReadTimeSeries()}

\idx{LALI2ReadTimeVectorSeries()}
\idx{LALI4ReadTimeVectorSeries()}
\idx{LALI8ReadTimeVectorSeries()}
\idx{LALU2ReadTimeVectorSeries()}
\idx{LALU4ReadTimeVectorSeries()}
\idx{LALU8ReadTimeVectorSeries()}
\idx{LALSReadTimeVectorSeries()}
\idx{LALDReadTimeVectorSeries()}
\idx{LALCReadTimeVectorSeries()}
\idx{LALZReadTimeVectorSeries()}

\idx{LALI2ReadTimeArraySeries()}
\idx{LALI4ReadTimeArraySeries()}
\idx{LALI8ReadTimeArraySeries()}
\idx{LALU2ReadTimeArraySeries()}
\idx{LALU4ReadTimeArraySeries()}
\idx{LALU8ReadTimeArraySeries()}
\idx{LALSReadTimeArraySeries()}
\idx{LALDReadTimeArraySeries()}
\idx{LALCReadTimeArraySeries()}
\idx{LALZReadTimeArraySeries()}

\idx{LALI2ReadFrequencySeries()}
\idx{LALI4ReadFrequencySeries()}
\idx{LALI8ReadFrequencySeries()}
\idx{LALU2ReadFrequencySeries()}
\idx{LALU4ReadFrequencySeries()}
\idx{LALU8ReadFrequencySeries()}
\idx{LALSReadFrequencySeries()}
\idx{LALDReadFrequencySeries()}
\idx{LALCReadFrequencySeries()}
\idx{LALZReadFrequencySeries()}

\subsubsection*{Description}

These routines parse an input stream \verb@*stream@ to fill in the
data and metadata fields of a time or frequency series \verb@*series@.
The field \verb@series->data@ must be \verb@NULL@, so that it can be
created and filled by the routine.  The other fields may be
initialized or not; they will be overwritten by metadata read from
\verb@*stream@.  If an error occurs, \verb@*series@ will be left
unchanged, but \verb@*stream@ will have been read up to the point
where the error occured.

For each of these prototype templates there are in fact 10 separate
routines corresponding to all the atomic datatypes \verb@<datatype>@
(except \verb@CHAR@) referred to by \verb@<typecode>@:
\begin{center}
\begin{tabular}{|c@{\qquad}c|c@{\qquad}c|}
\hline
\tt <typecode> & \tt <datatype> & \tt <typecode> & \tt <datatype> \\
\hline
\tt I2 & \tt  INT2 & \tt U2 & \tt    UINT2  \\
\tt I4 & \tt  INT4 & \tt U4 & \tt    UINT4  \\
\tt I8 & \tt  INT8 & \tt U8 & \tt    UINT8  \\
\tt  S & \tt REAL4 & \tt  C & \tt COMPLEX8  \\
\tt  D & \tt REAL8 & \tt  Z & \tt COMPLEX16 \\
\hline
\end{tabular}
\end{center}

\paragraph{Format for \texttt{*stream}:} The input stream is assumed
to be a text stream (ASCII) consisting of a header containing metadata
followed by numerical data in standard integer or floating-point
format, as recognized by the routines in \verb@StringConvert.c@.  The
header consists of zero or more lines beginning with a \verb@'#'@
character, followed by a metadata field name and value in the format:

\medskip
\begin{tabular}{l}
\verb@# @\textit{fieldname}\verb@=@\textit{value}
\end{tabular}
\medskip

\noindent The \verb@=@ sign in this format is standard but optional;
it may be replaced or surrounded with any amount of any whitespace
except a newline \verb@'\n'@.  If \textit{fieldname} is unrecognized,
it is ignored; if it is recognized, then \textit{value} must be in a
suitable format for the field type, as described below.  Blank lines,
or lines containing just a \verb@#@ character, are skipped.  Once a
line is encountered that contains non-whitespace characters and does
not start with \verb@'#'@, that line is assumed to be the beginning of
the numerical data.  From that point on, all non-whitespace characters
must be part of parseable numbers; no more comments are permitted
(although blank lines will still be skipped).

If a metadata field appears twice in the header, the later one takes
precedence.  At present these routines do not track which fields have
been previously assigned, so no warnings or errors are generated.

How the data is packed into the \verb@series->data@ structure depends
on what metadata has been provided, as described below.

\paragraph{Required, conditional, and optional metadata:} The input
stream need not contain a complete set of metadata, allowing some
metadata to be read from \verb@*stream@ and others to be set
elsewhere.  For each type of series, some metadata will be
\emph{required}, and the routine will abort if the metadata is not
found.  Other metadata are \emph{conditional}, meaning that the
routine will operate differently depending on whether or not these
metadata were found.  The remaining metadata are \emph{optional}; if
they are not found in \verb@*stream@, they will be left unchanged.
The recognized metadata fields are listed below.

\medskip\noindent\verb@<datatype>TimeSeries@:
\begin{description}
\item[Required fields:] none
\item[Conditional fields:] \verb@length@
\item[Optional fields:] \verb@name@, \verb@epoch@, \verb@deltaT@,
\verb@f0@, \verb@sampleUnits@, \verb@datatype@
\end{description}

\medskip\noindent\verb@<datatype>TimeVectorSeries@:
\begin{description}
\item[Required fields:] none
\item[Conditional fields:] \verb@length@, \verb@vectorLength@
\item[Optional fields:] \verb@name@, \verb@epoch@, \verb@deltaT@,
\verb@f0@, \verb@sampleUnits@, \verb@datatype@
\end{description}

\medskip\noindent\verb@<datatype>TimeArraySeries@:
\begin{description}
\item[Required fields:] \verb@dimLength@
\item[Conditional fields:] \verb@length@, \verb@arrayDim@
\item[Optional fields:] \verb@name@, \verb@epoch@, \verb@deltaT@,
\verb@f0@, \verb@sampleUnits@, \verb@datatype@
\end{description}

\medskip\noindent\verb@<datatype>FrequencySeries@:
\begin{description}
\item[Required fields:] none
\item[Conditional fields:] \verb@length@
\item[Optional fields:] \verb@name@, \verb@epoch@, \verb@deltaT@,
\verb@f0@, \verb@deltaF@, \verb@sampleUnits@, \verb@datatype@
\end{description}

Below we describe the required format for the field values, as well as
what occurs if a conditional field is or isn't present.

\bigskip\noindent\textit{Required fields:}
\begin{description}
\item[\texttt{dimLength}] (\verb@TimeArraySeries@ only):
\textit{value} consists of a sequence of \verb@UINT4@s separated by
whitespace (but \emph{not} a newline \verb@'\n'@).  These data are
stored in \verb@series->data->dimLength@: the number of integers gives
the number of array indecies, while the value of each integer gives
the dimension of the corresponding array index.
\end{description}

\medskip\noindent\textit{Conditional fields:}
\begin{description}
\item[\texttt{arrayDim}] (\verb@TimeArraySeries@ only): \textit{value}
is a single \verb@UINT4@, to be stored in
\verb@series->data->arrayDim@.  This must equal the product of the
index ranges in \verb@dimLength@, above, or an error is returned.  If
not given, the \verb@arrayDim@ field will be set equal to the product
of the index ranges in \verb@dimLength@.  (The \verb@arrayDim@ and
\verb@dimLength@ fields can appear in any order in \verb@*stream@;
checking is done only after all header lines have been read.)

\item[\texttt{vectorLength}] (\verb@TimeVectorSeries@ only):
\textit{value} is a single \verb@UINT4@, to be stored in
\verb@series->data->vectorLength@.  If not specified in the header
portion of \verb@*stream@, it will be taken to be the number of data
on the \emph{first} line of the data portion of \verb@*stream@, or
half the number of real data for a complex-valued
\verb@TimeVectorSeries@; if an odd number of real data are found on
the first line of a complex \verb@TimeVectorSeries@, then an error is
returned.

\item[\texttt{length}:] \textit{value} is a single \verb@UINT4@, to be
stored in \verb@series->data->length@.  If it is specified in the
header portion of \verb@*stream@, data will be read until
\verb@length@ is reached.  Otherwise, \verb@*stream@ will be read to
its end or until an unparseable character is read, and \verb@length@
will then be set accordingly.  (If parsing stops in the middle of
filling a complex, vector, or array valued element, the partly-read
element is discarded.)
\end{description}

\medskip\noindent\textit{Optional fields:}
\begin{description}
\item[\texttt{name}:] \textit{value} is a string surrounded by quotes
\verb@"@, which is parsed in the manner of a string literal in C: it
may contain ordinary printable characters (except \verb@"@ and
\verb@\@), escape sequences (such as \verb@\t@ for tab, \verb@\n@ for
newline, or \verb@\\@ and \verb@\"@ for literal backslash and quote
characters), and octal or hexadecimal codes (\verb@\@$ooo$ or
\verb@\x@$hh$ respectively) for arbitrary bytes.  Unlike in C,
literals cannot be split between lines, adjacent literals are not
concatenated, and converted strings longer than
\verb@LALNameLength@$-1$ will be truncated.  The resulting string is
stored in \verb@series->name@, and will always contain a \verb@\0@
terminator, beyond which the contents are unspecified.

\item[\texttt{epoch}:] \textit{value} is a single \verb@INT8@ number
of GPS nanoseconds, or a pair of \verb@INT4@s representing GPS seconds
and nanoseconds separately, separated by non-newline whitespace.

\item[\texttt{deltaT}] (any time series): \textit{value} is a single
\verb@REAL8@ number.

\item[\texttt{f0}:] \textit{value} is a single \verb@REAL8@ number.

\item[\texttt{deltaF}] (\verb@FrequencySeries@ only): \textit{value}
is a single \verb@REAL8@ number.

\item[\texttt{sampleUnits}:] \textit{value} is string surrounded by
quotes \verb@"@; the quotes are stripped and the string passed to
\verb@LALParseUnitString()@ to determine \verb@series->sampleUnits@.
Since \verb@LALParseUnitString()@ is not very robust, it is
recommended to use only unit strings that have been generated by
\verb@LALUnitAsString()@, or to remove this metadata field and set
\verb@series->sampleUnits@ within the code.

\item[\texttt{datatype}:] \textit{value} is string identifying the
series type; e.g. \verb@REAL4TimeSeries@ (\emph{not} surrounded by
quotes).  This should correspond to the type of \verb@*series@, not to
any field in \verb@*series@.  If there is a type mismatch, a warning
is generated (and errors may occur later while parsing the data).
\end{description}

\paragraph{Data format:} The data portion of \verb@*stream@ consists
of whitespace-separated integer or real numbers.  For complex input
routines, the real data are parsed as alternately the real and
imaginary parts of successive complex numbers.  By convention, each
line should correspond to a single base, complex, vector, or array
valued element of the \verb@series->data@ sequence.  However, this is
\emph{required} only in the case of a \verb@TimeVectorSeries@ where
the \verb@vectorLength@ metadata was not set in the header, since in
this case the value of \verb@vectorLength@ will be taken from the
number of elements read on the first data line.  After this, and in
all other cases, newlines are treated as any other whitespace.

If a \verb@length@ value is specified in the header, then data are
read until the required length is acheived; if \verb@fscanf()@ returns
zero or negative before this (representing either the end-of-input or
a character that cannot be interpreted as part of the numerical data),
an error is returned.  If a \verb@length@ value was not specified,
data are read until \verb@fscanf()@ returns zero or negative: at this
point any partially-completed complex, vector, or array valued element
is discarded, and \verb@series->data->length@ set to the number of
elements read.

\subsubsection*{Algorithm}

These routines use \verb@LALCHARReadVector()@ to read the header lines
and the first line of data.  After this, data are parsed directly from
\verb@*stream@ using \verb@fscanf()@.  This is done for efficiency:
repeated calling of the LAL string parsing routines in
\verb@StringConvert.c@ involves far too much computational overhead.

After the first data line has been read, the length of each sequence
element will be known from the atomic type, as well as the specified
\verb@dimLength@ (for arrays), \verb@vectorLength@ (for vectors), or
number of elements on the first data line (for vectors without an
explicitly specified \verb@vectorLength@).  If \verb@length@ is also
specified, a sequence of the appropriate size is allocated, and all
the data is copied or read directly into it.  If \verb@length@ was not
specified, the data read with \verb@fscanf()@ are stored in a linked
list of buffers of size \verb@BUFFSIZE@ (a local \verb@#define@d
constant) until parsing stops.  Then a sequence of the appropriate
size is allocated and the data copied into it.

\subsubsection*{Uses}
\begin{verbatim}
lalDebugLevel
LALPrintError()                         LALWarning()
LALMalloc()                             LALFree()
LALCHARReadVector()                     LALCHARDestroyVector()
LAL<typecode>CreateVector()             LAL<typecode>DestroyVector()
LAL<typecode>CreateVectorSequence()     LAL<typecode>DestroyVectorSequence()
LAL<typecode>CreateArraySequence()      LAL<typecode>DestroyArraySequence()
LALStringTo<typecode>()                 LALParseUnitString()
\end{verbatim}
where \verb@<typecode>@ is any of \verb@I2@, \verb@I4@, \verb@I8@,
\verb@U2@, \verb@U4@, \verb@U8@, \verb@S@, \verb@D@, \verb@C@, or
\verb@Z@.

\subsubsection*{Notes}

\vfill{\footnotesize\input{StreamSeriesInputCV}}

% This quote will fix the C syntax highlighting: "

******************************************************* </lalLaTeX> */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/SeqFactories.h>
#include <lal/Units.h>
#include <lal/StringInput.h>
#include <lal/StreamInput.h>

NRCSID( STREAMSERIESINPUTC, "$Id$" );

/* Define a message string for header parsing errors. */
#define LALREADSERIESC_HEADER "Skipping badly-formatted line for metadata field "

/* Define linked-list of buffers for storing an arbitrary number of
   arbitrary datatypes.  BUFFSIZE should be a multiple of 16. */
#define BUFFSIZE 24
typedef union tagBuffer {
  INT2 I2[BUFFSIZE/2];
  INT4 I4[BUFFSIZE/4];
  INT8 I8[BUFFSIZE/8];
  UINT2 U2[BUFFSIZE/2];
  UINT4 U4[BUFFSIZE/4];
  UINT8 U8[BUFFSIZE/8];
  REAL4 S[BUFFSIZE/4];
  REAL8 D[BUFFSIZE/8];
} Buffer;
typedef struct tagBufferList {
  Buffer buf;
  struct tagBufferList *next;
} BufferList;

/* Define a macro for freeing the linked list. */
#define FREEBUFFERLIST( headPtr )                                    \
if ( headPtr ) {                                                     \
  BufferList *herePtr = headPtr;                                     \
  while ( herePtr ) {                                                \
    BufferList *nextPtr = herePtr->next;                             \
    LALFree( herePtr );                                              \
    herePtr = nextPtr;                                               \
  }                                                                  \
} else (void)(0)

/* Define a function for parsing a string literal. */
static void
LALLiteralToString( LALStatus  *stat,
		    CHAR       *string,
		    const CHAR *literal,
		    UINT4      length )
{
  CHAR c;       /* Current character being considered. */
  UINT4 n = 0;  /* Counter of number of characters written. */

  INITSTATUS( stat, "LALLiteralToString", STREAMSERIESINPUTC );

  /* Find open quote. */
  while ( ( c = *literal ) != '"' && c != '\n' && c != '\0' )
    literal++;
  if ( *literal != '"' ) {
    LALWarning( stat, "No open quote found" );
    RETURN( stat );
  }
  literal++;

  /* Start parsing. */
  while ( n < length - 1 ) {

    /* End of literal, either implicit or explicit. */
    if ( ( c = *(literal++) ) == '\0' || c == '\n' ) {
      LALWarning( stat, "No close quote found" );
      string[n] = '\0';
      RETURN( stat );
    } else if ( c == '"' ) {
      string[n] = '\0';
      RETURN( stat );
    }

    /* Escape sequence. */
    else if ( c == '\\' ) {

      /* Do not allow actual end-of-line or end-of-string to be
         escaped. */
      if ( ( c = *(literal++) ) == '\0' || c == '\n' ) {
	LALWarning( stat, "No close quote found" );
	string[n] = '\0';
	RETURN( stat );
      }

      /* Other special escape characters. */
      else if ( c == 'a' || c == 'A' )
	string[n++] = '\a';
      else if ( c == 'b' || c == 'B' )
	string[n++] = '\b';
      else if ( c == 'f' || c == 'F' )
	string[n++] = '\f';
      else if ( c == 'n' || c == 'N' )
	string[n++] = '\n';
      else if ( c == 'r' || c == 'R' )
	string[n++] = '\r';
      else if ( c == 't' || c == 'T' )
	string[n++] = '\t';
      else if ( c == 'v' || c == 'V' )
	string[n++] = '\v';

      /* Hexadecimal character code. */
      else if ( c == 'x' || c == 'X' ) {
	c = *(literal++);   /* first digit */
	if ( isxdigit( c ) ) {
	  UINT2 value;
	  if ( isdigit( c ) )
	    value = c - '0';
	  else
	    value = 10 + tolower( c ) - 'a';
	  c = *(literal++); /* second digit */
	  if ( isxdigit( c ) ) {
	    value *= 16;
	    if ( isdigit( c ) )
	      value += c - '0';
	    else
	      value += 10 + tolower( c ) - 'a';
	  } else            /* no second digit */
	    literal--;
	  string[n++] = (CHAR)( value );
	  if ( value == 0 ) {
	    LALWarning( stat, "Found explicit end-of-string \\0" );
	    RETURN( stat );
	  }
	} else {            /* no first digit */
	  LALWarning( stat, "Treating empty hex cde as explicit"
		      " end-of-string \\0" );
	  string[n] = '\0';
	  RETURN( stat );
	}
      }

      /* Octal character code. */
      else if ( c >= '0' && c < '8' ) {
	UINT2 value = c - '0';
	c = *(literal++);   /* second digit */
	if ( c >= '0' && c < '8' ) {
	  value *= 8;
	  value += c - '0';
	  c = *(literal++); /* third digit */
	  if ( c >= '0' && c < '8' ) {
	    value *= 8;
	    value += c - '0';
	  } else            /* no third digit */
	    literal--;
	} else              /* no second digit */
	  literal--;
	if ( value > 255 )
	  LALWarning( stat, "Ignoring octal character code >= '\\400'" );
	else
	  string[n++] = (CHAR)( value );
	if ( value == 0 ) {
	  LALWarning( stat, "Found explicit end-of-string \\0" );
	  RETURN( stat );
	}
      }

      /* Other escaped character. */
      else {
	if ( c != '\\' && c != '?' && c != '\'' && c != '"' )
	  LALWarning( stat, "Dropping \\ from unrecognized escape"
		      " sequence" );
	string[n++] = c;
      }
    }

    /* Other character. */
    else
      string[n++] = c;
  }

  if ( *literal != '"' )
    LALWarning( stat, "Reached maximum length before reading close"
		" quote" );
  string[n] = '\0';
  RETURN( stat );
}

/* tell the GNU compiler to ignore issues with the `ll' length modifier */
#ifdef __GNUC__
#define fscanf __extension__ fscanf
#endif
/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI2ReadTSeries ( LALStatus *stat, INT2TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  INT2 *data = NULL;        /* pointer to data in buffers */
  INT2 *sData = NULL;       /* pointer to data in output sequence */
  INT2TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALI2ReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/2/2) == BUFFSIZE/2, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT2TimeSeries" ) ) {
	    LALWarning( stat, "INT2TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALI2CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToI2 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI2 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI2 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, sData++ ) != 1 ) {
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I2;
    n = BUFFSIZE/2;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToI2 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I2;
	length += n = BUFFSIZE/2;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_INT2_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/2 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I2;
	n = BUFFSIZE/2;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALI2CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.I2;
    sData = sCopy.data->data;
    n = BUFFSIZE/2;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I2;
      n = BUFFSIZE/2;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I2, n*sizeof( INT2 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I2;
      n = BUFFSIZE/2;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALI2ReadTVectorSeries ( LALStatus *stat, INT2TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  INT2 *data = NULL;        /* pointer to data in buffers */
  INT2 *sData = NULL;       /* pointer to data in output sequence */
  INT2TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALI2ReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/2/2) == BUFFSIZE/2, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT2TimeVectorSeries" ) ) {
	    LALWarning( stat, "INT2TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALI2CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 0
    do {
      LALStringToI2 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI2 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI2 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I2;
    n = BUFFSIZE/2;
    do {
      do {
	LALStringToI2 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I2;
	nTot += n = BUFFSIZE/2;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/2 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 0
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALI2CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.I2;
      sData = sCopy.data->data;
      n = BUFFSIZE/2;
      while ( vectorLength ) {
#if 0
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.I2;
	n = BUFFSIZE/2;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.I2, n*sizeof( INT2 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.I2;
	n = BUFFSIZE/2;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, sData++ ) != 1 ) {
	TRY( LALI2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_INT2_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.I2;
	length += n = BUFFSIZE/2;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 0
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALI2CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.I2;
    sData = sCopy.data->data;
    n = BUFFSIZE/2;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I2;
      n = BUFFSIZE/2;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I2, n*sizeof( INT2 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I2;
      n = BUFFSIZE/2;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALI2ReadTArraySeries ( LALStatus *stat, INT2TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  INT2 *data;        /* pointer to data in buffers */
  INT2 *sData;       /* pointer to data in output sequence */
  INT2TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALI2ReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/2/2) == BUFFSIZE/2, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT2TimeArraySeries" ) ) {
	    LALWarning( stat, "INT2TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALI2CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 0
    do {
      LALStringToI2 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI2 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI2 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, sData++ ) != 1 ) {
	TRY( LALI2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I2;
    n = BUFFSIZE/2;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToI2 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I2;
	length += n = BUFFSIZE/2;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_INT2_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/2 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I2;
	n = BUFFSIZE/2;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALI2CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.I2;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/2;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I2;
      n = BUFFSIZE/2;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I2, n*sizeof( INT2 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I2;
      n = BUFFSIZE/2;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI2ReadFSeries ( LALStatus *stat, INT2FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  INT2 *data;        /* pointer to data in buffers */
  INT2 *sData;       /* pointer to data in output sequence */
  INT2FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALI2ReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/2/2) == BUFFSIZE/2, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT2FrequencySeries" ) ) {
	    LALWarning( stat, "INT2FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALI2CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToI2 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI2 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI2 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT2_FORMAT, sData++ ) != 1 ) {
	TRY( LALI2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I2;
    n = BUFFSIZE/2;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToI2 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I2;
	length += n = BUFFSIZE/2;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_INT2_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/2 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I2;
	n = BUFFSIZE/2;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALI2CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.I2;
    sData = sCopy.data->data;
    n = BUFFSIZE/2;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I2;
      n = BUFFSIZE/2;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I2, n*sizeof( INT2 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I2;
      n = BUFFSIZE/2;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI4ReadTSeries ( LALStatus *stat, INT4TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  INT4 *data = NULL;        /* pointer to data in buffers */
  INT4 *sData = NULL;       /* pointer to data in output sequence */
  INT4TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALI4ReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT4TimeSeries" ) ) {
	    LALWarning( stat, "INT4TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALI4CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToI4 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI4 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI4 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, sData++ ) != 1 ) {
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I4;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToI4 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I4;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_INT4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I4;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALI4CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.I4;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I4;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I4, n*sizeof( INT4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I4;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALI4ReadTVectorSeries ( LALStatus *stat, INT4TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  INT4 *data = NULL;        /* pointer to data in buffers */
  INT4 *sData = NULL;       /* pointer to data in output sequence */
  INT4TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALI4ReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT4TimeVectorSeries" ) ) {
	    LALWarning( stat, "INT4TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALI4CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 0
    do {
      LALStringToI4 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI4 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI4 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I4;
    n = BUFFSIZE/4;
    do {
      do {
	LALStringToI4 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I4;
	nTot += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/4 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 0
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALI4CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.I4;
      sData = sCopy.data->data;
      n = BUFFSIZE/4;
      while ( vectorLength ) {
#if 0
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.I4;
	n = BUFFSIZE/4;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.I4, n*sizeof( INT4 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.I4;
	n = BUFFSIZE/4;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, sData++ ) != 1 ) {
	TRY( LALI4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_INT4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.I4;
	length += n = BUFFSIZE/4;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 0
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALI4CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.I4;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I4;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I4, n*sizeof( INT4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I4;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALI4ReadTArraySeries ( LALStatus *stat, INT4TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  INT4 *data;        /* pointer to data in buffers */
  INT4 *sData;       /* pointer to data in output sequence */
  INT4TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALI4ReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT4TimeArraySeries" ) ) {
	    LALWarning( stat, "INT4TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALI4CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 0
    do {
      LALStringToI4 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI4 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI4 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, sData++ ) != 1 ) {
	TRY( LALI4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I4;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToI4 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I4;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_INT4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I4;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALI4CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.I4;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I4;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I4, n*sizeof( INT4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I4;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI4ReadFSeries ( LALStatus *stat, INT4FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  INT4 *data;        /* pointer to data in buffers */
  INT4 *sData;       /* pointer to data in output sequence */
  INT4FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALI4ReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT4FrequencySeries" ) ) {
	    LALWarning( stat, "INT4FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALI4CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToI4 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI4 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI4 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT4_FORMAT, sData++ ) != 1 ) {
	TRY( LALI4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I4;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToI4 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I4;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_INT4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I4;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALI4CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.I4;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I4;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I4, n*sizeof( INT4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I4;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI8ReadTSeries ( LALStatus *stat, INT8TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  INT8 *data = NULL;        /* pointer to data in buffers */
  INT8 *sData = NULL;       /* pointer to data in output sequence */
  INT8TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALI8ReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT8TimeSeries" ) ) {
	    LALWarning( stat, "INT8TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALI8CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToI8 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI8 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI8 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, sData++ ) != 1 ) {
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I8;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToI8 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I8;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_INT8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I8;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALI8CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.I8;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I8;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I8, n*sizeof( INT8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I8;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALI8ReadTVectorSeries ( LALStatus *stat, INT8TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  INT8 *data = NULL;        /* pointer to data in buffers */
  INT8 *sData = NULL;       /* pointer to data in output sequence */
  INT8TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALI8ReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT8TimeVectorSeries" ) ) {
	    LALWarning( stat, "INT8TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALI8CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 0
    do {
      LALStringToI8 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI8 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI8 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I8;
    n = BUFFSIZE/8;
    do {
      do {
	LALStringToI8 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I8;
	nTot += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/8 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 0
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALI8CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.I8;
      sData = sCopy.data->data;
      n = BUFFSIZE/8;
      while ( vectorLength ) {
#if 0
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.I8;
	n = BUFFSIZE/8;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.I8, n*sizeof( INT8 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.I8;
	n = BUFFSIZE/8;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, sData++ ) != 1 ) {
	TRY( LALI8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_INT8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.I8;
	length += n = BUFFSIZE/8;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 0
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALI8CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.I8;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I8;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I8, n*sizeof( INT8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I8;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALI8ReadTArraySeries ( LALStatus *stat, INT8TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  INT8 *data;        /* pointer to data in buffers */
  INT8 *sData;       /* pointer to data in output sequence */
  INT8TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALI8ReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT8TimeArraySeries" ) ) {
	    LALWarning( stat, "INT8TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALI8CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 0
    do {
      LALStringToI8 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI8 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI8 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, sData++ ) != 1 ) {
	TRY( LALI8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I8;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToI8 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I8;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_INT8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I8;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALI8CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.I8;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I8;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I8, n*sizeof( INT8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I8;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI8ReadFSeries ( LALStatus *stat, INT8FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  INT8 *data;        /* pointer to data in buffers */
  INT8 *sData;       /* pointer to data in output sequence */
  INT8FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALI8ReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "INT8FrequencySeries" ) ) {
	    LALWarning( stat, "INT8FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALI8CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToI8 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToI8 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToI8 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_INT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_INT8_FORMAT, sData++ ) != 1 ) {
	TRY( LALI8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.I8;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToI8 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I8;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_INT8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.I8;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALI8CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.I8;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.I8;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.I8, n*sizeof( INT8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.I8;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU2ReadTSeries ( LALStatus *stat, UINT2TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  UINT2 *data = NULL;        /* pointer to data in buffers */
  UINT2 *sData = NULL;       /* pointer to data in output sequence */
  UINT2TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALU2ReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/2/2) == BUFFSIZE/2, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT2TimeSeries" ) ) {
	    LALWarning( stat, "UINT2TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALU2CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToU2 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU2 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU2 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, sData++ ) != 1 ) {
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U2;
    n = BUFFSIZE/2;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToU2 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U2;
	length += n = BUFFSIZE/2;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_UINT2_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/2 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U2;
	n = BUFFSIZE/2;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALU2CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.U2;
    sData = sCopy.data->data;
    n = BUFFSIZE/2;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U2;
      n = BUFFSIZE/2;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U2, n*sizeof( UINT2 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U2;
      n = BUFFSIZE/2;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALU2ReadTVectorSeries ( LALStatus *stat, UINT2TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT2 *data = NULL;        /* pointer to data in buffers */
  UINT2 *sData = NULL;       /* pointer to data in output sequence */
  UINT2TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALU2ReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/2/2) == BUFFSIZE/2, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT2TimeVectorSeries" ) ) {
	    LALWarning( stat, "UINT2TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALU2CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 0
    do {
      LALStringToU2 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU2 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU2 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U2;
    n = BUFFSIZE/2;
    do {
      do {
	LALStringToU2 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U2;
	nTot += n = BUFFSIZE/2;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/2 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 0
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALU2CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.U2;
      sData = sCopy.data->data;
      n = BUFFSIZE/2;
      while ( vectorLength ) {
#if 0
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.U2;
	n = BUFFSIZE/2;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.U2, n*sizeof( UINT2 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.U2;
	n = BUFFSIZE/2;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, sData++ ) != 1 ) {
	TRY( LALU2DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_UINT2_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.U2;
	length += n = BUFFSIZE/2;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 0
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALU2CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.U2;
    sData = sCopy.data->data;
    n = BUFFSIZE/2;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U2;
      n = BUFFSIZE/2;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U2, n*sizeof( UINT2 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U2;
      n = BUFFSIZE/2;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALU2ReadTArraySeries ( LALStatus *stat, UINT2TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  UINT2 *data;        /* pointer to data in buffers */
  UINT2 *sData;       /* pointer to data in output sequence */
  UINT2TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALU2ReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/2/2) == BUFFSIZE/2, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT2TimeArraySeries" ) ) {
	    LALWarning( stat, "UINT2TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALU2CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 0
    do {
      LALStringToU2 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU2 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU2 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, sData++ ) != 1 ) {
	TRY( LALU2DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U2;
    n = BUFFSIZE/2;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToU2 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U2;
	length += n = BUFFSIZE/2;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_UINT2_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/2 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U2;
	n = BUFFSIZE/2;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALU2CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.U2;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/2;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U2;
      n = BUFFSIZE/2;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U2, n*sizeof( UINT2 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U2;
      n = BUFFSIZE/2;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU2ReadFSeries ( LALStatus *stat, UINT2FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  UINT2 *data;        /* pointer to data in buffers */
  UINT2 *sData;       /* pointer to data in output sequence */
  UINT2FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALU2ReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/2/2) == BUFFSIZE/2, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT2FrequencySeries" ) ) {
	    LALWarning( stat, "UINT2FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALU2CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToU2 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU2 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU2 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT2_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT2_FORMAT, sData++ ) != 1 ) {
	TRY( LALU2DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U2;
    n = BUFFSIZE/2;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToU2 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U2;
	length += n = BUFFSIZE/2;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_UINT2_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/2 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U2;
	n = BUFFSIZE/2;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALU2CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.U2;
    sData = sCopy.data->data;
    n = BUFFSIZE/2;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U2;
      n = BUFFSIZE/2;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U2, n*sizeof( UINT2 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U2;
      n = BUFFSIZE/2;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU4ReadTSeries ( LALStatus *stat, UINT4TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  UINT4 *data = NULL;        /* pointer to data in buffers */
  UINT4 *sData = NULL;       /* pointer to data in output sequence */
  UINT4TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALU4ReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT4TimeSeries" ) ) {
	    LALWarning( stat, "UINT4TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALU4CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToU4 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU4 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU4 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, sData++ ) != 1 ) {
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U4;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToU4 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U4;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_UINT4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U4;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALU4CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.U4;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U4;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U4, n*sizeof( UINT4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U4;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALU4ReadTVectorSeries ( LALStatus *stat, UINT4TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 *data = NULL;        /* pointer to data in buffers */
  UINT4 *sData = NULL;       /* pointer to data in output sequence */
  UINT4TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALU4ReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT4TimeVectorSeries" ) ) {
	    LALWarning( stat, "UINT4TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALU4CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 0
    do {
      LALStringToU4 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU4 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU4 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U4;
    n = BUFFSIZE/4;
    do {
      do {
	LALStringToU4 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U4;
	nTot += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/4 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 0
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALU4CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.U4;
      sData = sCopy.data->data;
      n = BUFFSIZE/4;
      while ( vectorLength ) {
#if 0
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.U4;
	n = BUFFSIZE/4;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.U4, n*sizeof( UINT4 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.U4;
	n = BUFFSIZE/4;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, sData++ ) != 1 ) {
	TRY( LALU4DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_UINT4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.U4;
	length += n = BUFFSIZE/4;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 0
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALU4CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.U4;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U4;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U4, n*sizeof( UINT4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U4;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALU4ReadTArraySeries ( LALStatus *stat, UINT4TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  UINT4 *data;        /* pointer to data in buffers */
  UINT4 *sData;       /* pointer to data in output sequence */
  UINT4TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALU4ReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT4TimeArraySeries" ) ) {
	    LALWarning( stat, "UINT4TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALU4CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 0
    do {
      LALStringToU4 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU4 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU4 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, sData++ ) != 1 ) {
	TRY( LALU4DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U4;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToU4 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U4;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_UINT4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U4;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALU4CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.U4;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U4;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U4, n*sizeof( UINT4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U4;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU4ReadFSeries ( LALStatus *stat, UINT4FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  UINT4 *data;        /* pointer to data in buffers */
  UINT4 *sData;       /* pointer to data in output sequence */
  UINT4FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALU4ReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT4FrequencySeries" ) ) {
	    LALWarning( stat, "UINT4FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALU4CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToU4 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU4 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU4 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT4_FORMAT, sData++ ) != 1 ) {
	TRY( LALU4DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U4;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToU4 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U4;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_UINT4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U4;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALU4CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.U4;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U4;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U4, n*sizeof( UINT4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U4;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU8ReadTSeries ( LALStatus *stat, UINT8TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  UINT8 *data = NULL;        /* pointer to data in buffers */
  UINT8 *sData = NULL;       /* pointer to data in output sequence */
  UINT8TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALU8ReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT8TimeSeries" ) ) {
	    LALWarning( stat, "UINT8TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALU8CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToU8 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU8 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU8 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, sData++ ) != 1 ) {
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U8;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToU8 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U8;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_UINT8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U8;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALU8CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.U8;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U8;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U8, n*sizeof( UINT8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U8;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALU8ReadTVectorSeries ( LALStatus *stat, UINT8TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT8 *data = NULL;        /* pointer to data in buffers */
  UINT8 *sData = NULL;       /* pointer to data in output sequence */
  UINT8TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALU8ReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT8TimeVectorSeries" ) ) {
	    LALWarning( stat, "UINT8TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALU8CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 0
    do {
      LALStringToU8 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU8 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU8 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U8;
    n = BUFFSIZE/8;
    do {
      do {
	LALStringToU8 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U8;
	nTot += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/8 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 0
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALU8CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.U8;
      sData = sCopy.data->data;
      n = BUFFSIZE/8;
      while ( vectorLength ) {
#if 0
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.U8;
	n = BUFFSIZE/8;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.U8, n*sizeof( UINT8 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.U8;
	n = BUFFSIZE/8;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, sData++ ) != 1 ) {
	TRY( LALU8DestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_UINT8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.U8;
	length += n = BUFFSIZE/8;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 0
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALU8CreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.U8;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U8;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U8, n*sizeof( UINT8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U8;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALU8ReadTArraySeries ( LALStatus *stat, UINT8TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  UINT8 *data;        /* pointer to data in buffers */
  UINT8 *sData;       /* pointer to data in output sequence */
  UINT8TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALU8ReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT8TimeArraySeries" ) ) {
	    LALWarning( stat, "UINT8TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALU8CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 0
    do {
      LALStringToU8 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU8 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU8 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, sData++ ) != 1 ) {
	TRY( LALU8DestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U8;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToU8 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U8;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_UINT8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U8;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALU8CreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.U8;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U8;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U8, n*sizeof( UINT8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U8;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU8ReadFSeries ( LALStatus *stat, UINT8FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  UINT8 *data;        /* pointer to data in buffers */
  UINT8 *sData;       /* pointer to data in output sequence */
  UINT8FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALU8ReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "UINT8FrequencySeries" ) ) {
	    LALWarning( stat, "UINT8FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALU8CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToU8 ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToU8 ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToU8 ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_UINT8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_UINT8_FORMAT, sData++ ) != 1 ) {
	TRY( LALU8DestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.U8;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToU8 ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U8;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_UINT8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.U8;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALU8CreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.U8;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.U8;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.U8, n*sizeof( UINT8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.U8;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALSReadTSeries ( LALStatus *stat, REAL4TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  REAL4 *data = NULL;        /* pointer to data in buffers */
  REAL4 *sData = NULL;       /* pointer to data in output sequence */
  REAL4TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALSReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "REAL4TimeSeries" ) ) {
	    LALWarning( stat, "REAL4TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALSCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToS ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToS ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToS ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, sData++ ) != 1 ) {
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.S;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToS ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALSCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.S;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.S, n*sizeof( REAL4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALSReadTVectorSeries ( LALStatus *stat, REAL4TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  REAL4 *data = NULL;        /* pointer to data in buffers */
  REAL4 *sData = NULL;       /* pointer to data in output sequence */
  REAL4TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALSReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "REAL4TimeVectorSeries" ) ) {
	    LALWarning( stat, "REAL4TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALSCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 0
    do {
      LALStringToS ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALSDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToS ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALSDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToS ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALSDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.S;
    n = BUFFSIZE/4;
    do {
      do {
	LALStringToS ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	nTot += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/4 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 0
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALSCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.S;
      sData = sCopy.data->data;
      n = BUFFSIZE/4;
      while ( vectorLength ) {
#if 0
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.S;
	n = BUFFSIZE/4;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.S, n*sizeof( REAL4 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.S;
	n = BUFFSIZE/4;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALSDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALSDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, sData++ ) != 1 ) {
	TRY( LALSDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_REAL4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.S;
	length += n = BUFFSIZE/4;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 0
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALSCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.S;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.S, n*sizeof( REAL4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALSReadTArraySeries ( LALStatus *stat, REAL4TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  REAL4 *data;        /* pointer to data in buffers */
  REAL4 *sData;       /* pointer to data in output sequence */
  REAL4TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALSReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "REAL4TimeArraySeries" ) ) {
	    LALWarning( stat, "REAL4TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALSCreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 0
    do {
      LALStringToS ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALSDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToS ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALSDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToS ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALSDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALSDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALSDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, sData++ ) != 1 ) {
	TRY( LALSDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.S;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToS ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALSCreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.S;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.S, n*sizeof( REAL4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALSReadFSeries ( LALStatus *stat, REAL4FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  REAL4 *data;        /* pointer to data in buffers */
  REAL4 *sData;       /* pointer to data in output sequence */
  REAL4FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALSReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "REAL4FrequencySeries" ) ) {
	    LALWarning( stat, "REAL4FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALSCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToS ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToS ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToS ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, sData++ ) != 1 ) {
	TRY( LALSDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.S;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToS ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALSCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.S;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.S, n*sizeof( REAL4 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALDReadTSeries ( LALStatus *stat, REAL8TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  REAL8 *data = NULL;        /* pointer to data in buffers */
  REAL8 *sData = NULL;       /* pointer to data in output sequence */
  REAL8TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALDReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "REAL8TimeSeries" ) ) {
	    LALWarning( stat, "REAL8TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALDCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToD ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToD ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToD ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, sData++ ) != 1 ) {
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.D;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToD ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALDCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.D;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.D, n*sizeof( REAL8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALDReadTVectorSeries ( LALStatus *stat, REAL8TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  REAL8 *data = NULL;        /* pointer to data in buffers */
  REAL8 *sData = NULL;       /* pointer to data in output sequence */
  REAL8TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALDReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "REAL8TimeVectorSeries" ) ) {
	    LALWarning( stat, "REAL8TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALDCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 0
    do {
      LALStringToD ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALDDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToD ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALDDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToD ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALDDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.D;
    n = BUFFSIZE/8;
    do {
      do {
	LALStringToD ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	nTot += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/8 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 0
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALDCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.D;
      sData = sCopy.data->data;
      n = BUFFSIZE/8;
      while ( vectorLength ) {
#if 0
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.D;
	n = BUFFSIZE/8;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.D, n*sizeof( REAL8 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.D;
	n = BUFFSIZE/8;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALDDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALDDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, sData++ ) != 1 ) {
	TRY( LALDDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_REAL8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.D;
	length += n = BUFFSIZE/8;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 0
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALDCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.D;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.D, n*sizeof( REAL8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALDReadTArraySeries ( LALStatus *stat, REAL8TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  REAL8 *data;        /* pointer to data in buffers */
  REAL8 *sData;       /* pointer to data in output sequence */
  REAL8TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALDReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "REAL8TimeArraySeries" ) ) {
	    LALWarning( stat, "REAL8TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALDCreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 0
    do {
      LALStringToD ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALDDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToD ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALDDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToD ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALDDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALDDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALDDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, sData++ ) != 1 ) {
	TRY( LALDDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.D;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToD ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALDCreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.D;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.D, n*sizeof( REAL8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALDReadFSeries ( LALStatus *stat, REAL8FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  REAL8 *data;        /* pointer to data in buffers */
  REAL8 *sData;       /* pointer to data in output sequence */
  REAL8FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALDReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 0
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "REAL8FrequencySeries" ) ) {
	    LALWarning( stat, "REAL8FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALDCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 0
    do {
      LALStringToD ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToD ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToD ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 0
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, sData++ ) != 1 ) {
	TRY( LALDDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.D;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToD ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 0
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALDCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.D;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 0
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.D, n*sizeof( REAL8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALCReadTSeries ( LALStatus *stat, COMPLEX8TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  REAL4 *data = NULL;        /* pointer to data in buffers */
  COMPLEX8 *sData = NULL;       /* pointer to data in output sequence */
  COMPLEX8TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALCReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 1
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "COMPLEX8TimeSeries" ) ) {
	    LALWarning( stat, "COMPLEX8TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALCCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 1
    do {
      LALStringToS ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToS ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToS ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 1
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, sData++ ) != 1 ) {
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.S;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToS ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 1
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALCCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.S;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 1
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.S, n*sizeof( COMPLEX8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALCReadTVectorSeries ( LALStatus *stat, COMPLEX8TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  REAL4 *data = NULL;        /* pointer to data in buffers */
  COMPLEX8 *sData = NULL;       /* pointer to data in output sequence */
  COMPLEX8TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALCReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 1
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "COMPLEX8TimeVectorSeries" ) ) {
	    LALWarning( stat, "COMPLEX8TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALCCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 1
    do {
      LALStringToS ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToS ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALCDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToS ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.S;
    n = BUFFSIZE/4;
    do {
      do {
	LALStringToS ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	nTot += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/4 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 1
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALCCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.S;
      sData = sCopy.data->data;
      n = BUFFSIZE/4;
      while ( vectorLength ) {
#if 1
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.S;
	n = BUFFSIZE/4;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.S, n*sizeof( COMPLEX8 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.S;
	n = BUFFSIZE/4;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 1
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALCDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALCDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, sData++ ) != 1 ) {
	TRY( LALCDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_REAL4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.S;
	length += n = BUFFSIZE/4;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 1
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALCCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.S;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 1
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.S, n*sizeof( COMPLEX8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALCReadTArraySeries ( LALStatus *stat, COMPLEX8TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  REAL4 *data;        /* pointer to data in buffers */
  COMPLEX8 *sData;       /* pointer to data in output sequence */
  COMPLEX8TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALCReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 1
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "COMPLEX8TimeArraySeries" ) ) {
	    LALWarning( stat, "COMPLEX8TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALCCreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 1
    do {
      LALStringToS ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToS ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALCDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToS ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 1
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALCDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALCDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, sData++ ) != 1 ) {
	TRY( LALCDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.S;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToS ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 1
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALCCreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.S;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/4;
    while ( length ) {
#if 1
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.S, n*sizeof( COMPLEX8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALCReadFSeries ( LALStatus *stat, COMPLEX8FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  REAL4 *data;        /* pointer to data in buffers */
  COMPLEX8 *sData;       /* pointer to data in output sequence */
  COMPLEX8FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALCReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 1
  ASSERT( 2*(BUFFSIZE/4/2) == BUFFSIZE/4, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "COMPLEX8FrequencySeries" ) ) {
	    LALWarning( stat, "COMPLEX8FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALCCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 1
    do {
      LALStringToS ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToS ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToS ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 1
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL4_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL4_FORMAT, sData++ ) != 1 ) {
	TRY( LALCDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.S;
    n = BUFFSIZE/4;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToS ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	length += n = BUFFSIZE/4;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL4_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/4 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.S;
	n = BUFFSIZE/4;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 1
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALCCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.S;
    sData = sCopy.data->data;
    n = BUFFSIZE/4;
    while ( length ) {
#if 1
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.S, n*sizeof( COMPLEX8 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.S;
      n = BUFFSIZE/4;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALZReadTSeries ( LALStatus *stat, COMPLEX16TimeSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  REAL8 *data = NULL;        /* pointer to data in buffers */
  COMPLEX16 *sData = NULL;       /* pointer to data in output sequence */
  COMPLEX16TimeSeries sCopy; /* internal copy of series */
  int numRead = 0; /* number of values read by parsing subroutine */

  INITSTATUS( stat, "LALZReadTSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT,
	  STREAMINPUTH_MSGEOUT );
#if 1
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "COMPLEX16TimeSeries" ) ) {
	    LALWarning( stat, "COMPLEX16TimeSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALZCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 1
    do {
      LALStringToD ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToD ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToD ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 1
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) ) {
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, sData++ ) != 1 ) {
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.D;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToD ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 1
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALZCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.D;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 1
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.D, n*sizeof( COMPLEX16 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALZReadTVectorSeries ( LALStatus *stat, COMPLEX16TimeVectorSeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head = NULL;  /* pointer to head of linked list of buffers */
  BufferList *here = NULL;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 vectorLength = 0; /* number of components per element */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  REAL8 *data = NULL;        /* pointer to data in buffers */
  COMPLEX16 *sData = NULL;       /* pointer to data in output sequence */
  COMPLEX16TimeVectorSeries sCopy; /* internal copy of series */
  CreateVectorSequenceIn in; /* structure to create sequence */
  int numRead = 0; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALZReadTVectorSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 1
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse vectorLength field. */
      else if ( !strcmp( start, "vectorLength" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "vectorLength" );
	else
	  vectorLength = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "COMPLEX16TimeVectorSeries" ) ) {
	    LALWarning( stat, "COMPLEX16TimeVectorSeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;
  in.length = length;
  in.vectorLength = vectorLength;

  /* If length and vectorLength were both specified, we know we will
     need length*vectorLength data, so we can just allocate this space
     from the start. */
  if ( length > 0 && vectorLength > 0 ) {
    LALZCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Read the line in memory. */
    n = length*vectorLength;
    numRead = 0;
#if 1
    do {
      LALStringToD ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALZDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToD ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALZDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToD ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALZDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif
  }

  /* Otherwise, we will have to read at least the entire first line
     currently in memory. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.D;
    n = BUFFSIZE/8;
    do {
      do {
	LALStringToD ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	nTot += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );
  }
  nTot += BUFFSIZE/8 - n;

  /* We can now compute vectorLength, if it wasn't specified. */
  if ( in.vectorLength == 0 ) {
    in.vectorLength = vectorLength = nTot;
    if ( in.vectorLength == 0 ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
#if 1
    vectorLength /= 2;
    if ( 2*vectorLength < in.vectorLength ) {
      FREEBUFFERLIST( head );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EVLEN, STREAMINPUTH_MSGEVLEN );
    }
    in.vectorLength = vectorLength;
#endif

    /* If length was specified, we can now create the vector sequence,
       and copy the first line into it. */
    if ( length > 0 ) {
      LALZCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
      BEGINFAIL( stat ) {
	FREEBUFFERLIST( head );
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      } ENDFAIL( stat );
      here = head;
      data = here->buf.D;
      sData = sCopy.data->data;
      n = BUFFSIZE/8;
      while ( vectorLength ) {
#if 1
	n /= 2;
	if ( n > vectorLength )
	  n = vectorLength;
	vectorLength -= n;
	while ( n-- ) {
	  sData->re = *(data++);
	  (sData++)->im = *(data++);
	}
	here = here->next;
	if ( here )
	  data = here->buf.D;
	n = BUFFSIZE/8;
#else
	if ( n > vectorLength )
	  n = vectorLength;
	memcpy( sData, here->buf.D, n*sizeof( COMPLEX16 ) );
	sData += n;
	vectorLength -= n;
	here = here->next;
	if ( here )
	  data = here->buf.D;
	n = BUFFSIZE/8;
#endif
      }
      FREEBUFFERLIST( head );
      n = ( length - 1 )*in.vectorLength;
      numRead = 0;
    }
  }

  /* Read the remaining data with fscanf().  If length was specified,
     then the data sequence has already been created, and n is the
     number of atomic (integer, floating-point, or complex) data still
     to be read. */
  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
  if ( length > 0 ) {
#if 1
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALZDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALZDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, sData++ ) != 1 ) {
	TRY( LALZDestroyVectorSequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }

  /* Otherwise, if length wasn't specified, we need to continue
     reading into buffers.  In this case n is the space remaining in
     the current buffer. */
  else {
    length = nTot + n;
    while ( numRead == 1 ) {
      do
	numRead = fscanf( stream, "%" LAL_REAL8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length -= n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !(here->next) ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here = here->next;
	here->next = NULL;
	data = here->buf.D;
	length += n = BUFFSIZE/8;
      }
    }

    /* Now create the data sequence and copy data into it. */
#if 1
    length /= 2;
#endif
    in.length = length/in.vectorLength;
    length = in.length*in.vectorLength;
    LALZCreateVectorSequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
    } ENDFAIL( stat );
    here = head;
    data = here->buf.D;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 1
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.D, n*sizeof( COMPLEX16 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/*<lalVerbatim file="StreamSeriesInputCP"> */
void
LALZReadTArraySeries ( LALStatus *stat, COMPLEX16TimeArraySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 nTot = 0, n; /* number of data read, and countdown index */
  UINT4 arrayDim = 0; /* number of components per element */
  UINT4Vector *dimLength = NULL; /* number of components per index */
  REAL8 *data;        /* pointer to data in buffers */
  COMPLEX16 *sData;       /* pointer to data in output sequence */
  COMPLEX16TimeArraySeries sCopy; /* internal copy of series */
  CreateArraySequenceIn in; /* structure to create sequence */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALZReadTArraySeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 1
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse deltaT field. */
      else if ( !strcmp( start, "deltaT" ) ) {
	REAL8 deltaT;
	LALStringToD( stat->statusPtr, &deltaT, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaT" );
	else
	  sCopy.deltaT = deltaT;
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	    if ( dimLength ) {
	      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	    }
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse arrayDim field. */
      else if ( !strcmp( start, "arrayDim" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "arrayDim" );
	else
	  arrayDim = tempLength;
      }

      /* Parse dimLength field. */
      else if ( !strcmp( start, "dimLength" ) ) {
	UINT4 *dData;
	here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
	if ( !here ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	dData = here->buf.U4;
	n = BUFFSIZE/4;
	endValue = startValue;
	/* Read components into the buffer list. */
	do {
	  do {
	    LALStringToU4( stat->statusPtr, dData++,
			   startValue = endValue, &endValue );
	    BEGINFAIL( stat ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	    } ENDFAIL( stat );
	  } while ( endValue != startValue && --n );
	  if ( !n ) {
	    here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	    here = here->next;
	    if ( !here ) {
	      FREEBUFFERLIST( head );
	      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	      if ( dimLength ) {
		TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		     stat );
	      }
	      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	    }
	    here->next = NULL;
	    dData = here->buf.U4;
	    nTot += n = BUFFSIZE/4;
	  } else
	    nTot += BUFFSIZE/4 - n;
	} while ( endValue != startValue );
	/* Copy components into dimLength vector. */
	if ( nTot == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "dimLength" );
	else {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ),
		 stat );
	  }
	  dimLength = NULL;
	  LALU4CreateVector( stat->statusPtr, &dimLength, nTot );
	  BEGINFAIL( stat ) {
	    FREEBUFFERLIST( head );
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  here = head;
	  dData = dimLength->data;
	  n = BUFFSIZE/4;
	  if ( n > nTot )
	    n = nTot;
	  while ( n ) {
	    memcpy( dData, here->buf.U4, n*sizeof(UINT4) );
	    dData += n;
	    nTot -= n;
	    here = here->next;
	    n = BUFFSIZE/4;
	    if ( n > nTot )
	      n = nTot;
	  }
	}
	/* dimLength is complete, so reset everything else. */
	FREEBUFFERLIST( head );
	nTot = 0;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "COMPLEX16TimeArraySeries" ) ) {
	    LALWarning( stat, "COMPLEX16TimeArraySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    LALCHARReadVector( stat->statusPtr, &line, stream );
    BEGINFAIL( stat ) {
      if ( dimLength ) {
	TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      }
    } ENDFAIL( stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	LALCHARReadVector( stat->statusPtr, &line, stream );
	BEGINFAIL( stat ) {
	  if ( dimLength ) {
	    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
	  }
	} ENDFAIL( stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* Make sure dimLength exists, and is consistent with arrayDim (if
     specified). */
  if ( !dimLength ) {
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    ABORT( stat, STREAMINPUTH_EDLEN, STREAMINPUTH_MSGEDLEN );
  } else {
    UINT4 i;       /* index over array indecies */
    UINT4 dim = 1; /* total array outer-product dimension */
    for ( i = 0; i < dimLength->length; i++ )
      dim *= dimLength->data[i];
    if ( dim == 0 || ( arrayDim != 0 && arrayDim != dim ) ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      ABORT( stat, STREAMINPUTH_EDIM, STREAMINPUTH_MSGEDIM );
    }
    arrayDim = dim;
  }

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    in.length = length;
    in.dimLength = dimLength;
    LALZCreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length*arrayDim;
    numRead = 0;
#if 1
    do {
      LALStringToD ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALZDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToD ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALZDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToD ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALZDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 1
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALZDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALZDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, sData++ ) != 1 ) {
	TRY( LALZDestroyArraySequence ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.D;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToD ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
    length /= arrayDim;
#if 1
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    in.length = length;
    in.dimLength = dimLength;
    LALZCreateArraySequence ( stat->statusPtr, &(sCopy.data), &in );
    BEGINFAIL( stat ) {
      FREEBUFFERLIST( head );
      TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    } ENDFAIL( stat );
    TRY( LALU4DestroyVector( stat->statusPtr, &dimLength ), stat );
    here = head;
    data = here->buf.D;
    sData = sCopy.data->data;
    length *= arrayDim;
    n = BUFFSIZE/8;
    while ( length ) {
#if 1
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.D, n*sizeof( COMPLEX16 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALZReadFSeries ( LALStatus *stat, COMPLEX16FrequencySeries *series, FILE *stream )
{ /* </lalVerbatim> */
  BufferList *head;  /* pointer to head of linked list of buffers */
  BufferList *here;  /* pointer to current position in list */
  CHARVector *line = NULL; /* current line being read */
  CHAR *start, *end; /* start and end of a token on a line */
  UINT4 length = 0;  /* number of sequence elements to be read */
  UINT4 n;           /* countdown index over elements read */
  REAL8 *data;        /* pointer to data in buffers */
  COMPLEX16 *sData;       /* pointer to data in output sequence */
  COMPLEX16FrequencySeries sCopy; /* internal copy of series */
  int numRead; /* number of values read per call of fscanf() */

  INITSTATUS( stat, "LALZReadFSeries", STREAMSERIESINPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( series, stat, STREAMINPUTH_ENUL, STREAMINPUTH_MSGENUL );
  ASSERT( !(series->data), stat, STREAMINPUTH_EOUT, STREAMINPUTH_MSGEOUT );
#if 1
  ASSERT( 2*(BUFFSIZE/8/2) == BUFFSIZE/8, stat,
	  STREAMINPUTH_EBUF, STREAMINPUTH_MSGEBUF );
#endif

  /*******************************************************************
   * PARSE METADATA HEADER                                           *
   *******************************************************************/

  sCopy = *series;

  /* Skip over blank lines; start points to the first non-whitespace
     character (or '\0' if there are none). */
  TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
  start = line->data;
  while ( isspace( *start ) ) {
    start++;
    if ( *start == '\0' ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
      start = line->data;
    }
  }

  /* As long as '#' is the first non-whitespace character of any
     nonblank lines... */
  while ( *start == '#' ) {
    CHAR *startValue; /* start of substring giving metadata value */
    CHAR *endValue;   /* end of substring giving metadata value */

    /* Skip to the start of the metadata field tag. */
    do
      start++;
    while ( isspace( *start ) );

    /* Mark the end of the tag and the start of the metadata value. */
    end = start;
    while ( !isspace( *end ) && *end != '=' && *end != '\0' )
      end++;
    startValue = end;
    while ( isspace( *startValue ) || *startValue == '=' )
      startValue++;
    if ( startValue != end ) {
      *end = '\0';

      /* Parse name field. */
      if ( !strcmp( start, "name" ) ) {
	if ( *startValue != '"' )
	  LALWarning( stat, LALREADSERIESC_HEADER "name" );
	else
	  LALLiteralToString( stat->statusPtr, sCopy.name, startValue,
			      LALNameLength );
      }

      /* Parse epoch field. */
      else if ( !strcmp( start, "epoch" ) ) {
	INT8 sec, nsec;
	LALStringToI8( stat->statusPtr, &sec, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "epoch" );
	else {
	  CHAR *temp = endValue;
	  LALStringToI8( stat->statusPtr, &nsec, endValue, &endValue );
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  if ( endValue == temp ) {
	    sCopy.epoch.gpsSeconds = (INT4)( sec/1000000000LL );
	    sCopy.epoch.gpsNanoSeconds = (INT4)
	      ( sec - 1000000000LL*sCopy.epoch.gpsSeconds );
	  } else {
	    sCopy.epoch.gpsSeconds = (INT4)( sec );
	    sCopy.epoch.gpsNanoSeconds = (INT4)( nsec );
	  }
	}
      }

      /* Parse f0 field. */
      else if ( !strcmp( start, "f0" ) ) {
	REAL8 f0;
	LALStringToD( stat->statusPtr, &f0, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "f0" );
	else
	  sCopy.f0 = f0;
      }

      /* Parse deltaF field. */
      else if ( !strcmp( start, "deltaF" ) ) {
	REAL8 deltaF;
	LALStringToD( stat->statusPtr, &deltaF, startValue, &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue )
	  LALWarning( stat, LALREADSERIESC_HEADER "deltaF" );
	else
	  sCopy.deltaF = deltaF;
      }

      /* Parse sampleUnits field. */
      else if ( !strcmp( start, "sampleUnits" ) ) {
	LALUnit unit;
	CHARVector unitString;
	endValue = ++startValue;
	while ( *endValue != '"' && *endValue != '\n' &&
		*endValue != '\0' )
	  endValue++;
	*endValue = '\0';
	unitString.length = strlen( startValue ) + 1;
	unitString.data = startValue;
	LALParseUnitString( stat->statusPtr, &unit, &unitString );
	if ( stat->statusPtr->statusCode == UNITSH_EPARSE ) {
#ifndef NDEBUG
	  if ( lalDebugLevel & LALERROR ) {
	    LALPrintError( "\tCONTINUE: Ignoring preceding error\n" );
	    DETATCHSTATUSPTR( stat );
	    ATTATCHSTATUSPTR( stat );
	  }
#endif
	  LALWarning( stat, LALREADSERIESC_HEADER "sampleUnits" );
	} else {
	  BEGINFAIL( stat ) {
	    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  } ENDFAIL( stat );
	  sCopy.sampleUnits = unit;
	}
      }

      /* Parse length field. */
      else if ( !strcmp( start, "length" ) ) {
	UINT4 tempLength;
	LALStringToU4( stat->statusPtr, &tempLength, startValue,
		       &endValue );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
	if ( endValue == startValue || tempLength == 0 )
	  LALWarning( stat, LALREADSERIESC_HEADER "length" );
	else
	  length = tempLength;
      }

      /* Parse datatype field. */
#ifndef NDEBUG
      else if ( !strcmp( start, "datatype" ) ) {
	if ( lalDebugLevel & LALWARNING ) {
	  endValue = startValue;
	  while ( !isspace( *endValue ) && *endValue != '\0' )
	    endValue++;
	  *endValue = '\0';
	  if ( strcmp( startValue, "COMPLEX16FrequencySeries" ) ) {
	    LALWarning( stat, "COMPLEX16FrequencySeries data expected" );
	    LALPrintError( "\t%s data being read\n", startValue );
	  }
	}
      }
#endif

      /* No other recognized tags; ignore anything else. */
    }

    /* Read in next line, skipping over blank lines. */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
    start = line->data;
    while ( isspace( *start ) ) {
      start++;
      if ( *start == '\0' ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALCHARReadVector( stat->statusPtr, &line, stream ), stat );
	start = line->data;
      }
    }
  }


  /*******************************************************************
   * PARSE DATA                                                      *
   *******************************************************************/

  end = start;

  /* If length is given, we can simply allocate the required block of
     memory and parse data into it. */
  if ( length > 0 ) {
    LALZCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    } ENDFAIL( stat );
    sData = sCopy.data->data;

    /* Begin with line in memory. */
    n = length;
    numRead = 0;
#if 1
    do {
      LALStringToD ( stat->statusPtr, &(sData->re), start = end,
		     &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
      if ( start != end ) {
	numRead = 1;
	LALStringToD ( stat->statusPtr, &(sData->im), start = end,
		       &end );
	BEGINFAIL( stat ) {
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	} ENDFAIL( stat );
	if ( start != end ) {
	  numRead = 0;
	  sData++;
	}
      }
    } while ( end != start && --n );
#else
    do {
      LALStringToD ( stat->statusPtr, sData++, start = end, &end );
      BEGINFAIL( stat ) {
	TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
      } ENDFAIL( stat );
    } while ( end != start && --n );
    sData--;
#endif

    /* Read remaining data with fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
#if 1
    if ( numRead == 1 ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
      n--;
    }
    while ( n-- ) {
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->re) ) != 1 ||
	   fscanf( stream, "%" LAL_REAL8_FORMAT, &(sData->im) ) != 1 ) {
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
      sData++;
    }
#else
    while ( n-- )
      if ( fscanf( stream, "%" LAL_REAL8_FORMAT, sData++ ) != 1 ) {
	TRY( LALZDestroyVector ( stat->statusPtr, &(sCopy.data) ), stat );
	ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
      }
#endif
  }


  /* If length was not specified, we must read data into buffers until
     we know how many there are. */
  else {
    here = head = (BufferList *)LALMalloc( sizeof(BufferList) );
    if ( !here ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
      ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
    }
    here->next = NULL;
    data = here->buf.D;
    n = BUFFSIZE/8;

    /* Read from the line currently in memory. */
    do {
      do {
	LALStringToD ( stat->statusPtr, data++, start = end, &end );
	BEGINFAIL( stat ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	} ENDFAIL( stat );
      } while ( end != start && --n );
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	length += n = BUFFSIZE/8;
      } else
	data--;
    } while ( end != start );

    /* Read the remaining data using fscanf(). */
    TRY( LALCHARDestroyVector( stat->statusPtr, &line ), stat );
    do {
      do
	numRead = fscanf( stream, "%" LAL_REAL8_FORMAT, data++ );
      while ( numRead == 1 && --n );
      length += BUFFSIZE/8 - n;
      if ( !n ) {
	here->next = (BufferList *)LALMalloc( sizeof(BufferList) );
	here = here->next;
	if ( !here ) {
	  FREEBUFFERLIST( head );
	  ABORT( stat, STREAMINPUTH_EMEM, STREAMINPUTH_MSGEMEM );
	}
	here->next = NULL;
	data = here->buf.D;
	n = BUFFSIZE/8;
      }
    } while ( numRead == 1 );

    /* Create the data sequence, and copy the data into it. */
#if 1
    length /= 2;
#endif
    if ( !length ) {
      FREEBUFFERLIST( head );
      ABORT( stat, STREAMINPUTH_ESLEN, STREAMINPUTH_MSGESLEN );
    }
    LALZCreateVector ( stat->statusPtr, &(sCopy.data), length );
    BEGINFAIL( stat )
      FREEBUFFERLIST( head );
    ENDFAIL( stat );

    here = head;
    data = here->buf.D;
    sData = sCopy.data->data;
    n = BUFFSIZE/8;
    while ( length ) {
#if 1
      n /= 2;
      if ( n > length )
	n = length;
      length -= n;
      while ( n-- ) {
	sData->re = *(data++);
	(sData++)->im = *(data++);
      }
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#else
      if ( n > length )
	n = length;
      memcpy( sData, here->buf.D, n*sizeof( COMPLEX16 ) );
      sData += n;
      length -= n;
      here = here->next;
      if ( here )
	data = here->buf.D;
      n = BUFFSIZE/8;
#endif
    }
    FREEBUFFERLIST( head );
  }

  /* Data have been stored successfully.  So, clean up and exit. */
  *series = sCopy;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}
