/*************************** <lalVerbatim file="StreamSeriesOutputCV">
Author: Creighton, T. D.
$Id$
**************************************************** </lalVerbatim> */

/********************************************************** <lalLaTeX>

\subsection{Module \texttt{StreamSeriesOutput.c}}
\label{ss:StreamSeriesOutput.c}

Writes a time or frequency series to an output stream.

\subsubsection*{Prototypes}
\vspace{0.1in}
\begin{verbatim}
void
LAL<typecode>WriteTSeries( LALStatus            *stat,
                           FILE                 *stream,
                           <datatype>TimeSeries *series )

void
LAL<typecode>WriteTVectorSeries( LALStatus                  *stat,
                                 FILE                       *stream,
                                 <datatype>TimeVectorSeries *series )

void
LAL<typecode>WriteTArraySeries( LALStatus                 *stat,
                                FILE                      *stream,
                                <datatype>TimeArraySeries *series )

void
LAL<typecode>WriteFSeries( LALStatus                 *stat,
                           FILE                      *stream,
                           <datatype>FrequencySeries *series )
\end{verbatim}

\idx{LALI2WriteTSeries()}
\idx{LALI4WriteTSeries()}
\idx{LALI8WriteTSeries()}
\idx{LALU2WriteTSeries()}
\idx{LALU4WriteTSeries()}
\idx{LALU8WriteTSeries()}
\idx{LALSWriteTSeries()}
\idx{LALDWriteTSeries()}
\idx{LALCWriteTSeries()}
\idx{LALZWriteTSeries()}

\idx{LALI2WriteTVectorSeries()}
\idx{LALI4WriteTVectorSeries()}
\idx{LALI8WriteTVectorSeries()}
\idx{LALU2WriteTVectorSeries()}
\idx{LALU4WriteTVectorSeries()}
\idx{LALU8WriteTVectorSeries()}
\idx{LALSWriteTVectorSeries()}
\idx{LALDWriteTVectorSeries()}
\idx{LALCWriteTVectorSeries()}
\idx{LALZWriteTVectorSeries()}

\idx{LALI2WriteTArraySeries()}
\idx{LALI4WriteTArraySeries()}
\idx{LALI8WriteTArraySeries()}
\idx{LALU2WriteTArraySeries()}
\idx{LALU4WriteTArraySeries()}
\idx{LALU8WriteTArraySeries()}
\idx{LALSWriteTArraySeries()}
\idx{LALDWriteTArraySeries()}
\idx{LALCWriteTArraySeries()}
\idx{LALZWriteTArraySeries()}

\idx{LALI2WriteFSeries()}
\idx{LALI4WriteFSeries()}
\idx{LALI8WriteFSeries()}
\idx{LALU2WriteFSeries()}
\idx{LALU4WriteFSeries()}
\idx{LALU8WriteFSeries()}
\idx{LALSWriteFSeries()}
\idx{LALDWriteFSeries()}
\idx{LALCWriteFSeries()}
\idx{LALZWriteFSeries()}

\subsubsection*{Description}

These routines write the data and metadata in a time or frequency
series \verb@*series@ to an output stream \verb@*stream@ in a standard
format, described below.  It returns an error if any attempt to write
to the stream failed; \verb@*stream@ may then be left in a
partially-written state.

For each of these prototype templates there are in fact 10 separate
routines corresponding to all the atomic datatypes \verb@<datatype>@
(except \verb@CHAR@) referred to by \verb@<typecode>@:
\begin{center}
\begin{tabular}{|c@{\qquad}c|c@{\qquad}c|}
\hline
\tt <typecode> & \tt <datatype> & \tt <typecode> & \tt <datatype> \\
\hline
\tt I2 & \tt  INT2 & \tt U2 & \tt    UINT2  \\
\tt I4 & \tt  INT4 & \tt U4 & \tt    UINT4  \\
\tt I8 & \tt  INT8 & \tt U8 & \tt    UINT8  \\
\tt  S & \tt REAL4 & \tt  C & \tt COMPLEX8  \\
\tt  D & \tt REAL8 & \tt  Z & \tt COMPLEX16 \\
\hline
\end{tabular}
\end{center}

\paragraph{Format for \texttt{*stream}:} The data written to the
output stream will be formatted in a manner consistent with the input
routines in \verb@StreamSeriesInput.c@.  That is, it will begin with a
metadata header, consisting of multiple lines of the form:

\medskip
\begin{tabular}{l}
\verb@# @\textit{fieldname}\verb@ = @\textit{value}
\end{tabular}
\medskip

\noindent where \textit{fieldname} is the name of a field in
\verb@*series@ and \textit{value} is the value of that metadata field,
in some standard format (below).  The following metadata fields will
be written, one per line, based on the type of \verb@*series@:

\begin{description}
\item[\texttt{<datatype>TimeSeries}:] \verb@datatype@, \verb@name@,
\verb@epoch@, \verb@deltaT@, \verb@f0@, \verb@sampleUnits@,
\verb@length@
\item[\texttt{<datatype>TimeVectorSeries}:] \verb@datatype@,
\verb@name@, \verb@epoch@, \verb@deltaT@, \verb@f0@,
\verb@sampleUnits@, \verb@length@, \verb@vectorLength@
\item[\texttt{<datatype>TimeArraySeries}:] \verb@datatype@,
\verb@name@, \verb@epoch@, \verb@deltaT@, \verb@f0@,
\verb@sampleUnits@, \verb@length@, \verb@dimLength@, \verb@arrayDim@
\item[\texttt{<datatype>FrequencySeries}:] \verb@datatype@,
\verb@name@, \verb@epoch@, \verb@deltaT@, \verb@f0@, \verb@deltaF@,
\verb@sampleUnits@, \verb@length@
\end{description}

\noindent After all metadata have been written, the contents of
\verb@series->data->data@ will be written in standard integer or
floating-point notation, according to \verb@<datatype>@: integers will
be written to full precision, while floating-point numbers will be
written in exponential notation with sufficient digits to ensure that
they represent a unique binary floating-point number under the IEEE
Standard 754 (this means 9 digits for \verb@REAL4@s and 17 digits for
\verb@REAL8@s).  Complex datatypes are represented by pairs of
floating-point numbers representing alternately the real and imaginary
parts.

The body of the file will be formatted with newlines \verb@'\n'@
separating individual base, complex, vector, or array valued elements
of the sequence \verb@series->data@.  Within each element, integer or
floating-point components will be separated by single \verb@' '@
characters.  Thus the value of \verb@series->data->length@ will always
equal the number of lines following the metadata header.

\paragraph{Format for metadata fields:} Here we summarize briefly the
format for the individual field values in the metadata header.

\begin{description}
\item[\texttt{datatype}:] \textit{value} is a string (\emph{not}
surrounded by quotes) corresponding to the type of \verb@*series@;
e.g.\ \verb@COMPLEX8FrequencySeries@.

\item[\texttt{name}:] \textit{value} is a string surrounded by quotes
\verb@"@ representing \verb@series->name@.  Standard C-language string
literal notation is used: printable characters are written directly
except for \verb@"@ and \verb@\@ (rendered as \verb@\"@ and \verb@\\@,
respectively), characters with special C escape sequences are written
as those sequences (e.g.\ \verb@\t@ for tab and \verb@\n@ for
newline), and all other character bytes are written as three-digit
octal codes \verb@\@$ooo$.  Writing stops at the first null byte
\verb@\0@.

\item[\texttt{epoch}:] \textit{value} is a single \verb@INT8@ number
representing \verb@series->epoch@ in GPS nanoseconds.

\item[\texttt{deltaT}] (any time series): \textit{value} is a single
\verb@REAL8@ number representing \verb@series->deltaT@.

\item[\texttt{f0}:] \textit{value} is a single \verb@REAL8@ number
representing \verb@series->f0@.

\item[\texttt{deltaF}] (\verb@FrequencySeries@ only): \textit{value}
is a single \verb@REAL8@ number representing \verb@series->deltaF@.

\item[\texttt{sampleUnits}:] \textit{value} is string surrounded by
quotes \verb@"@; inside the quotes is a unit string corresponding to
\verb@series->sampleUnits@ as converted by the routine
\verb@LALUnitAsString()@.

\item[\texttt{length}:] \textit{value} is a single \verb@UINT4@
representing \verb@series->data->length@.

\item[\texttt{vectorLength}] (\verb@TimeVectorSeries@ only):
\textit{value} is a single \verb@UINT4@ representing
\verb@series->data->vectorLength@.

\item[\texttt{dimLength}] (\verb@TimeArraySeries@ only):
\textit{value} consists of a sequence of \verb@UINT4@s separated by
single \verb@' '@ characters, representing the components of
\verb@series->data->dimLength->data@.  The value of
\verb@series->data->dimLength->length@ must be inferred from the
number of components; it is not given as separate metadata.

\item[\texttt{arrayDim}] (\verb@TimeArraySeries@ only): \textit{value}
is a single \verb@UINT4@ representing \verb@series->data->arrayDim@.
If the array sequence was properly constructed, this will equal the
product of the components of \verb@dimLength@, above.
\end{description}

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
lalDebugLevel                           LALPrintError()
LALCHARCreateVector()                   LALCHARDestroyVector()
LALUnitAsString()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{StreamSeriesOutputCV}}

******************************************************* </lalLaTeX> */

#include <stdio.h>
#include <ctype.h>
#include <lal/LALStdlib.h>
#include <lal/Units.h>
#include <lal/AVFactories.h>
#include <lal/StringInput.h>
#include <lal/StreamOutput.h>
#include <lal/Date.h>

NRCSID( STREAMSERIESOUTPUTC, "$Id$" );

/* Define a function for printing a string as a literal. */
static int
LALWriteLiteral( FILE *stream, const CHAR *string )
{
  CHAR c; /* Current character being considered. */

  /* Open literal and start parsing string. */
  if ( putc( '"', stream ) == EOF ) return 1;
  while ( ( c = *(string++) ) != '\0' ) {

    /* Characters with named escape sequences. */
    if ( c == '\a' ) {
      if ( fputs( "\\a", stream ) == EOF ) return 1;
    } else if ( c == '\b' ) {
      if ( fputs( "\\b", stream ) == EOF ) return 1;
    } else if ( c == '\f' ) {
      if ( fputs( "\\f", stream ) == EOF ) return 1;
    } else if ( c == '\n' ) {
      if ( fputs( "\\n", stream ) == EOF ) return 1;
    } else if ( c == '\r' ) {
      if ( fputs( "\\r", stream ) == EOF ) return 1;
    } else if ( c == '\t' ) {
      if ( fputs( "\\t", stream ) == EOF ) return 1;
    } else if ( c == '\v' ) {
      if ( fputs( "\\v", stream ) == EOF ) return 1;
    } else if ( c == '"' ) {
      if ( fputs( "\\\"", stream ) == EOF ) return 1;
    } else if ( c == '\\' ) {
      if ( fputs( "\\\\", stream ) == EOF ) return 1;
    }

    /* Printable characters. */
    else if ( isprint( c ) ) {
      if ( putc( c, stream ) == EOF ) return 1;
    }

    /* Other characters are given 3-digit octal escape sequences. */
    else {
      if ( fprintf( stream, "\\%03o", (UCHAR)( c ) ) < 0 )
	return 1;
    }
  }

  /* Close literal and exit. */
  if ( putc( '"', stream ) == EOF ) return 1;
  return 0;
}

/* tell GNU C compiler to ignore warnings about the `ll' length modifier */
#ifdef __GNUC__
#define fprintf __extension__ fprintf
#endif

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI2WriteTSeries ( LALStatus *stat, FILE *stream, INT2TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  INT2 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALI2WriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT2TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% " LAL_INT2_FORMAT " " "% " LAL_INT2_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT2_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI2WriteTVectorSeries ( LALStatus *stat, FILE *stream, INT2TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  INT2 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALI2WriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT2TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 0
    if ( fprintf( stream, "% " LAL_INT2_FORMAT " " "% " LAL_INT2_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT2_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% " LAL_INT2_FORMAT " " "% " LAL_INT2_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% " LAL_INT2_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI2WriteTArraySeries ( LALStatus *stat, FILE *stream, INT2TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  INT2 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALI2WriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT2TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 0
    if ( fprintf( stream, "% " LAL_INT2_FORMAT " " "% " LAL_INT2_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT2_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% " LAL_INT2_FORMAT " " "% " LAL_INT2_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% " LAL_INT2_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI2WriteFSeries ( LALStatus *stat, FILE *stream, INT2FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  INT2 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALI2WriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT2FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% " LAL_INT2_FORMAT " " "% " LAL_INT2_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT2_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI4WriteTSeries ( LALStatus *stat, FILE *stream, INT4TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  INT4 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALI4WriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT4TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% " LAL_INT4_FORMAT " " "% " LAL_INT4_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT4_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI4WriteTVectorSeries ( LALStatus *stat, FILE *stream, INT4TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  INT4 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALI4WriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT4TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 0
    if ( fprintf( stream, "% " LAL_INT4_FORMAT " " "% " LAL_INT4_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT4_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% " LAL_INT4_FORMAT " " "% " LAL_INT4_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% " LAL_INT4_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI4WriteTArraySeries ( LALStatus *stat, FILE *stream, INT4TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  INT4 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALI4WriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT4TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 0
    if ( fprintf( stream, "% " LAL_INT4_FORMAT " " "% " LAL_INT4_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT4_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% " LAL_INT4_FORMAT " " "% " LAL_INT4_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% " LAL_INT4_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI4WriteFSeries ( LALStatus *stat, FILE *stream, INT4FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  INT4 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALI4WriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT4FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% " LAL_INT4_FORMAT " " "% " LAL_INT4_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT4_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI8WriteTSeries ( LALStatus *stat, FILE *stream, INT8TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  INT8 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALI8WriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT8TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% " LAL_INT8_FORMAT " " "% " LAL_INT8_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT8_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI8WriteTVectorSeries ( LALStatus *stat, FILE *stream, INT8TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  INT8 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALI8WriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT8TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 0
    if ( fprintf( stream, "% " LAL_INT8_FORMAT " " "% " LAL_INT8_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT8_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% " LAL_INT8_FORMAT " " "% " LAL_INT8_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% " LAL_INT8_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI8WriteTArraySeries ( LALStatus *stat, FILE *stream, INT8TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  INT8 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALI8WriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT8TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 0
    if ( fprintf( stream, "% " LAL_INT8_FORMAT " " "% " LAL_INT8_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT8_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% " LAL_INT8_FORMAT " " "% " LAL_INT8_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% " LAL_INT8_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALI8WriteFSeries ( LALStatus *stat, FILE *stream, INT8FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  INT8 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALI8WriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = INT8FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% " LAL_INT8_FORMAT " " "% " LAL_INT8_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% " LAL_INT8_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU2WriteTSeries ( LALStatus *stat, FILE *stream, UINT2TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  UINT2 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALU2WriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT2TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "%" LAL_UINT2_FORMAT " " "%" LAL_UINT2_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT2_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU2WriteTVectorSeries ( LALStatus *stat, FILE *stream, UINT2TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  UINT2 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALU2WriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT2TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 0
    if ( fprintf( stream, "%" LAL_UINT2_FORMAT " " "%" LAL_UINT2_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT2_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "%" LAL_UINT2_FORMAT " " "%" LAL_UINT2_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "%" LAL_UINT2_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU2WriteTArraySeries ( LALStatus *stat, FILE *stream, UINT2TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  UINT2 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALU2WriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT2TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 0
    if ( fprintf( stream, "%" LAL_UINT2_FORMAT " " "%" LAL_UINT2_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT2_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "%" LAL_UINT2_FORMAT " " "%" LAL_UINT2_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "%" LAL_UINT2_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU2WriteFSeries ( LALStatus *stat, FILE *stream, UINT2FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  UINT2 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALU2WriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT2FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "%" LAL_UINT2_FORMAT " " "%" LAL_UINT2_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT2_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU4WriteTSeries ( LALStatus *stat, FILE *stream, UINT4TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  UINT4 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALU4WriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT4TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "%" LAL_UINT4_FORMAT " " "%" LAL_UINT4_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT4_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU4WriteTVectorSeries ( LALStatus *stat, FILE *stream, UINT4TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  UINT4 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALU4WriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT4TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 0
    if ( fprintf( stream, "%" LAL_UINT4_FORMAT " " "%" LAL_UINT4_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT4_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "%" LAL_UINT4_FORMAT " " "%" LAL_UINT4_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "%" LAL_UINT4_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU4WriteTArraySeries ( LALStatus *stat, FILE *stream, UINT4TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  UINT4 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALU4WriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT4TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 0
    if ( fprintf( stream, "%" LAL_UINT4_FORMAT " " "%" LAL_UINT4_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT4_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "%" LAL_UINT4_FORMAT " " "%" LAL_UINT4_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "%" LAL_UINT4_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU4WriteFSeries ( LALStatus *stat, FILE *stream, UINT4FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  UINT4 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALU4WriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT4FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "%" LAL_UINT4_FORMAT " " "%" LAL_UINT4_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT4_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU8WriteTSeries ( LALStatus *stat, FILE *stream, UINT8TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  UINT8 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALU8WriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT8TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "%" LAL_UINT8_FORMAT " " "%" LAL_UINT8_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT8_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU8WriteTVectorSeries ( LALStatus *stat, FILE *stream, UINT8TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  UINT8 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALU8WriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT8TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 0
    if ( fprintf( stream, "%" LAL_UINT8_FORMAT " " "%" LAL_UINT8_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT8_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "%" LAL_UINT8_FORMAT " " "%" LAL_UINT8_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "%" LAL_UINT8_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU8WriteTArraySeries ( LALStatus *stat, FILE *stream, UINT8TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  UINT8 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALU8WriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT8TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 0
    if ( fprintf( stream, "%" LAL_UINT8_FORMAT " " "%" LAL_UINT8_FORMAT, data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT8_FORMAT, *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "%" LAL_UINT8_FORMAT " " "%" LAL_UINT8_FORMAT, data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "%" LAL_UINT8_FORMAT, *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALU8WriteFSeries ( LALStatus *stat, FILE *stream, UINT8FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  UINT8 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALU8WriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = UINT8FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "%" LAL_UINT8_FORMAT " " "%" LAL_UINT8_FORMAT "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "%" LAL_UINT8_FORMAT "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALSWriteTSeries ( LALStatus *stat, FILE *stream, REAL4TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  REAL4 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALSWriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = REAL4TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% .8e" " " "% .8e" "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .8e" "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALSWriteTVectorSeries ( LALStatus *stat, FILE *stream, REAL4TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  REAL4 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALSWriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = REAL4TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 0
    if ( fprintf( stream, "% .8e" " " "% .8e", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .8e", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% .8e" " " "% .8e", data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% .8e", *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALSWriteTArraySeries ( LALStatus *stat, FILE *stream, REAL4TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  REAL4 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALSWriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = REAL4TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 0
    if ( fprintf( stream, "% .8e" " " "% .8e", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .8e", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% .8e" " " "% .8e", data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% .8e", *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALSWriteFSeries ( LALStatus *stat, FILE *stream, REAL4FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  REAL4 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALSWriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = REAL4FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% .8e" " " "% .8e" "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .8e" "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALDWriteTSeries ( LALStatus *stat, FILE *stream, REAL8TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  REAL8 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALDWriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = REAL8TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% .16e" " " "% .16e" "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .16e" "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALDWriteTVectorSeries ( LALStatus *stat, FILE *stream, REAL8TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  REAL8 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALDWriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = REAL8TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 0
    if ( fprintf( stream, "% .16e" " " "% .16e", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .16e", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% .16e" " " "% .16e", data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% .16e", *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALDWriteTArraySeries ( LALStatus *stat, FILE *stream, REAL8TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  REAL8 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALDWriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = REAL8TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 0
    if ( fprintf( stream, "% .16e" " " "% .16e", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .16e", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 0
      if ( fprintf( stream, " " "% .16e" " " "% .16e", data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% .16e", *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALDWriteFSeries ( LALStatus *stat, FILE *stream, REAL8FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  REAL8 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALDWriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = REAL8FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 0
    if ( fprintf( stream, "% .16e" " " "% .16e" "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .16e" "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALCWriteTSeries ( LALStatus *stat, FILE *stream, COMPLEX8TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  COMPLEX8 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALCWriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = COMPLEX8TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 1
    if ( fprintf( stream, "% .8e" " " "% .8e" "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .8e" "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALCWriteTVectorSeries ( LALStatus *stat, FILE *stream, COMPLEX8TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  COMPLEX8 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALCWriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = COMPLEX8TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 1
    if ( fprintf( stream, "% .8e" " " "% .8e", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .8e", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 1
      if ( fprintf( stream, " " "% .8e" " " "% .8e", data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% .8e", *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALCWriteTArraySeries ( LALStatus *stat, FILE *stream, COMPLEX8TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  COMPLEX8 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALCWriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = COMPLEX8TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 1
    if ( fprintf( stream, "% .8e" " " "% .8e", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .8e", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 1
      if ( fprintf( stream, " " "% .8e" " " "% .8e", data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% .8e", *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALCWriteFSeries ( LALStatus *stat, FILE *stream, COMPLEX8FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  COMPLEX8 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALCWriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = COMPLEX8FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 1
    if ( fprintf( stream, "% .8e" " " "% .8e" "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .8e" "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALZWriteTSeries ( LALStatus *stat, FILE *stream, COMPLEX16TimeSeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  COMPLEX16 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALZWriteTSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = COMPLEX16TimeSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 1
    if ( fprintf( stream, "% .16e" " " "% .16e" "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .16e" "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALZWriteTVectorSeries ( LALStatus *stat, FILE *stream, COMPLEX16TimeVectorSeries *series )
{ /* </lalVerbatim> */
  UINT4 length;       /* length of data sequence */
  UINT4 vectorLength; /* length of each element in sequence */
  COMPLEX16 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALZWriteTVectorSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = COMPLEX16TimeVectorSeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length and vectorLength. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# vectorLength = %" LAL_UINT4_FORMAT "\n",
		series->data->vectorLength ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  vectorLength = series->data->vectorLength;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = vectorLength - 1;
#if 1
    if ( fprintf( stream, "% .16e" " " "% .16e", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .16e", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 1
      if ( fprintf( stream, " " "% .16e" " " "% .16e", data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% .16e", *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALZWriteTArraySeries ( LALStatus *stat, FILE *stream, COMPLEX16TimeArraySeries *series )
{ /* </lalVerbatim> */
  UINT4 length;   /* length of data sequence */
  UINT4 arrayDim; /* length of each element in sequence */
  UINT4 *dimData; /* pointer to dimLength data */
  COMPLEX16 *data; /* pointer to data in sequence */

  INITSTATUS( stat, "LALZWriteTArraySeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->dimLength->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = COMPLEX16TimeArraySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaT, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaT = %.16e\n", series->deltaT ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length, dimLength, and arrayDim. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ||
       fprintf( stream, "# arrayDim = %" LAL_UINT4_FORMAT "\n",
		series->data->arrayDim ) < 0 ||
       fprintf( stream, "# dimLength =" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }
  arrayDim = series->data->dimLength->length;
  dimData = series->data->dimLength->data;
  while ( arrayDim-- )
    if ( fprintf( stream, " %" LAL_UINT4_FORMAT, *(dimData++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  if ( fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  arrayDim = series->data->arrayDim;
  data = series->data->data;
  while ( length-- ) {
    UINT4 n = arrayDim - 1;
#if 1
    if ( fprintf( stream, "% .16e" " " "% .16e", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .16e", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
    while ( n-- ) {
#if 1
      if ( fprintf( stream, " " "% .16e" " " "% .16e", data->re, data->im ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
      data++;
#else
      if ( fprintf( stream, " " "% .16e", *(data++) ) < 0 ) {
	ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
      }
#endif
    }
    if ( fprintf( stream, "\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}

/* <lalVerbatim file="StreamSeriesInputCP"> */
void
LALZWriteFSeries ( LALStatus *stat, FILE *stream, COMPLEX16FrequencySeries *series )
{ /* </lalVerbatim> */
  UINT4 length; /* length of data sequence */
  COMPLEX16 *data;   /* pointer to data in sequence */

  INITSTATUS( stat, "LALZWriteFSeries", STREAMSERIESOUTPUTC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( stream, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );
  ASSERT( series->data->data, stat, STREAMOUTPUTH_ENUL, STREAMOUTPUTH_MSGENUL );


  /*******************************************************************
   * PRINT METADATA HEADER                                           *
   *******************************************************************/

  /* Print the datatype. */
  if ( fprintf( stream, "# datatype = COMPLEX16FrequencySeries\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the name. */
  if ( fprintf( stream, "# name = " ) < 0 ||
       LALWriteLiteral( stream, series->name ) ||
       fprintf( stream, "\n" ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the epoch, deltaF, and f0. */
  if ( fprintf( stream, "# epoch = %" LAL_INT8_FORMAT "\n",
                XLALGPSToINT8NS( &series->epoch ) ) < 0 ||
       fprintf( stream, "# deltaF = %.16e\n", series->deltaF ) < 0 ||
       fprintf( stream, "# f0 = %.16e\n", series->f0 ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }

  /* Print the sample units. */
  {
    CHARVector *unit = NULL; /* vector to store unit string */
    CHAR *cData;             /* pointer to unit data */
    CHAR c;                  /* value of *cData */
    int code = 0;            /* return code from putc() */
    BOOLEAN repeat;          /* whether a longer vector is needed */

    /* First, use LALUnitAsString() to generate unit string.  Repeat
       if a longer vector is required. */
    length = 64;
    TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
    memset( unit->data, 0, length*sizeof(CHAR) );
    do {
      LALUnitAsString( stat->statusPtr, unit, &(series->sampleUnits) );
      if ( stat->statusPtr->statusCode == UNITSH_ESTRINGSIZE ) {
	repeat = 1;
	length *= 2;
	TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	TRY( LALCHARCreateVector( stat->statusPtr, &unit, length ), stat );
	memset( unit->data, 0, length*sizeof(CHAR) );
#ifndef NDEBUG
	if ( lalDebugLevel & LALERROR )
	  LALPrintError( "\tCONTINUE: Dealt with preceding error\n" );
#endif
      } else {
	repeat = 0;
	BEGINFAIL( stat )
	  TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
	ENDFAIL( stat );
      }
    } while ( repeat );

    /* Write the resulting unit string enclosed in quotes. */
    cData = unit->data;
    if ( fprintf( stream, "# sampleUnits = \"" ) < 0 ) {
      TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    while ( code != EOF && ( c = *(cData++) ) != '\0' && length-- )
      code = putc( (int)c, stream );
    TRY( LALCHARDestroyVector( stat->statusPtr, &unit ), stat );
    if ( code == EOF || fprintf( stream, "\"\n" ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
  }

  /* Print the length. */
  if ( fprintf( stream, "# length = %" LAL_UINT4_FORMAT "\n",
		series->data->length ) < 0 ) {
    ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
  }


  /*******************************************************************
   * PRINT DATA                                                      *
   *******************************************************************/

  length = series->data->length;
  data = series->data->data;
  while ( length-- ) {
#if 1
    if ( fprintf( stream, "% .16e" " " "% .16e" "\n", data->re, data->im ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
    data++;
#else
    if ( fprintf( stream, "% .16e" "\n", *(data++) ) < 0 ) {
      ABORT( stat, STREAMOUTPUTH_EPRN, STREAMOUTPUTH_MSGEPRN );
    }
#endif
  }
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}
